#include "math.inc"
#include "transforms.inc"
#include "colors.inc"
#include "woods.inc"
#include "stones1.inc"
#include "textures.inc"
#include "skies.inc"

#declare Seed = seed(1);

// 1m = 1 unit

sky_sphere { S_Cloud1 }

light_source {
   200*(y-z-x)
   color 1
#ifdef (DELUX)
   area_light <10, 0, 0>, <0, 0, 10>, 10, 10
   adaptive 1
   jitter
#end
}

// WATTLE & DAUB

#declare T_WattleDaub = texture {
   pigment { rgb <1, .8, .65> } normal { granite .1 }
}
texture {
   pigment { rgbt <.7, .8, .65, .75> } normal { granite .8 scale 10 }
}

#declare T_MyWood8 = texture { T_Wood8 normal { granite .8 scale 8 } }

#declare T_MyWood8rot = texture { T_MyWood8 rotate 2*x }

#declare T_RoughWood = texture {
   gradient z warp { turbulence .025 }
   texture_map {
      [ 0 T_MyWood8 ]
      [ .5 T_MyWood8rot ]
      [ 1 T_MyWood8 ]
   }
   rotate 89*x
   scale .1
}

#declare T_Reet = texture {
   Yellow_Pine
   rotate 90 * y
   rotate 4 * (y-x)
}
texture {
   pigment { rgbt <1,1,1,.7> }
   normal { bumps 1 scale 1-.95*z }
}
texture {
   pigment { rgbt <1,1,1,.85> }
   normal { bumps 1 rotate 45*z }
}

#declare P_Grass = pigment {
   bumps scale 1.5
   pigment_map {
      [0 granite color_map {[.3 color <.6,.6,.3>] [.6 color <.3,.5,.2>]} rotate 45*y]
      [.3 granite color_map {[.3 color <.3,.5,.2>] [.6 color <.5,.6,.1>]} rotate 20*y]
      [.6 granite color_map {[.3 color <.5,.6,.1>] [.6 color <.4,.5,.2>]}]
      [1 granite color_map {[.3 color <.4,.5,.2>] [.6 color <.6,.6,.3>]} rotate -30*y]
   }
   scale <.015, .015, .1>
   warp { turbulence 1 }
}

#declare T_Grass = texture {
   pigment { P_Grass }
   normal { pigment_pattern { P_Grass } 1.0 }
   scale .2*3
}

#declare T_Road = texture { T_Grnt27 scale .25*.2 }

#declare T_Water = texture {
   pigment { rgb .2 } 
   normal { crackle .25 scale <4.5, 10, 1> 
      turbulence 1.7
   }
   finish { ambient .2 diffuse .2 phong .1 reflection .6 }
   scale .2
}

#declare T_Sand = texture { T_Grnt4 scale .5*.2 }
texture { pigment { rgbt <1, 1, 0, .75> } }

#macro Beam(l)
      box {
         -1, 1 scale <.1, l + .1, .1>
         texture { T_RoughWood }
      }
#end

#macro SmallBeam(l)
      box {
         -1, 1 scale <.1, l + .05, .05>
         texture { T_RoughWood }
      }
#end

#declare O_RoofEnd = union {
   #local X = 2;
   #local Z = 50;
   #local i = -X;
   #while (i < X)
      #local i = i + 1;
      #local j = -Z;
      #while (j < Z)
         #local j = j + 1;
         prism {
            conic_sweep
            0, 1, 5,
            <1,1>, <-1,1>, <-1,-1>, <1,-1>, <1,1>
            translate <2*i-1, -1, 2*j-1>
            scale <.05/X, .2*rand(Seed), .5/Z>
            rotate 90*z
         }
      #end
   #end
}

// w = width, h = height, d = depth, ds = door side (counter clockwise
// starting from -z = 0), dw = door width, dh, = door height, rh = roof height,
// wp = window period, ww = window width, wh = window height

#macro House(w, h, d, ds, dw, dh, rh, wpx, wpy, wpz, ww, wh)
#ifndef (DELUX)
   union {
      box { <-1, 0, -1> <1, 1, 1> pigment { White } }
      box { <-1.05, 0, -1.05> <-.95, 1, -.95> pigment { MediumWood } }
      box { <-1.05, 0,  1.05> <-.95, 1,  .95> pigment { MediumWood } }
      box { < 1.05, 0, -1.05> < .95, 1, -.95> pigment { MediumWood } }
      box { < 1.05, 0,  1.05> < .95, 1,  .95> pigment { MediumWood } }
      prism { -1.15, 1.15, 4, <1, -1.15>, <1.15, 0>, <1, 1.15>, <1, -1.15>
         rotate 90*z pigment { LightWood }
      }
      scale <w/2, h+dh, d/2>
   }
#else
   union {
      // ** Wood structure
      // Vertical beams
      #local iw = int(w/4);
      #local i = -iw;
      #while (i <= iw)
         object { Beam(h) translate <-i*w/(2*iw), 0 , -d/2> }
         object { Beam(h) translate <-i*w/(2*iw), 0 , d/2> }
         #local i = i + 1;
      #end

      #local id = int(d/4);
      #local i = -id;
      #while (i <= id)
         object { Beam(h) translate <-w/2, 0, i*d/(2*id)> }
         object { Beam(h) translate <w/2, 0, i*d/(2*id)> }
         #local i = i + 1;
      #end

      // Cross beams
      #local ih = int(h);
      #local i = ih;
      #while (i > 0)
          object { Beam(d/2) rotate 90*x translate <-w/2, i*h/ih , 0> }
          object { Beam(d/2) rotate 90*x translate <w/2, i*h/ih , 0> }

          object { Beam(w/2) rotate 90*z translate <0, i*h/ih , -d/2> }
          object { Beam(w/2) rotate 90*z translate <0, i*h/ih , d/2> }
         #local i = i - 1;
      #end

      // ** Windows
      #local i = ih;
      #while (i > 0)
         #if (wpy != 0 & mod(i, wpy) = 0)
            // x-sides
            #local j = -id;
            #while (j < id)
               #if (wpz != 0 & mod(j, wpz) = 0)
                  union {
                     object { SmallBeam(wh/2) translate -ww/2*z }
                     object { SmallBeam(wh/2) translate ww/2*z }
                     object { SmallBeam(ww/2) rotate 90*x translate wh/2*y }
                     object { SmallBeam(ww/2) rotate 90*x translate -wh/2*y }
                     translate <w/2, i*h/ih - h/(2*ih) , j*d/(2*id) + d/(4*id)>
                  }
                  union {
                     object { SmallBeam(wh/2) translate -ww/2*z }
                     object { SmallBeam(wh/2) translate ww/2*z }
                     object { SmallBeam(ww/2) rotate 90*x translate wh/2*y }
                     object { SmallBeam(ww/2) rotate 90*x translate -wh/2*y }
                     translate <-w/2, i*h/ih - h/(2*ih) , j*d/(2*id) + d/(4*id)>
                  }
               #end
               #local j = j + 1;
            #end

            // z-sides
            #local j = -iw;
            #while (j < iw)
               #if ((wpx != 0) & (mod(j, wpx) = 0))
                  union {
                     object { SmallBeam(wh/2) translate -ww/2*z }
                     object { SmallBeam(wh/2) translate ww/2*z }
                     object { SmallBeam(ww/2) rotate 90*x translate wh/2*y }
                     object { SmallBeam(ww/2) rotate 90*x translate -wh/2*y }
                     rotate 90*y
                     translate <j*w/(2*iw) + w/(4*iw), i*h/ih - h/(2*ih) , d/2>
                  }
                  union {
                     object { SmallBeam(wh/2) translate -ww/2*z }
                     object { SmallBeam(wh/2) translate ww/2*z }
                     object { SmallBeam(ww/2) rotate 90*x translate wh/2*y }
                     object { SmallBeam(ww/2) rotate 90*x translate -wh/2*y }
                     rotate 90*y
                     translate <j*w/(2*iw) + w/(4*iw), i*h/ih - h/(2*ih) , -d/2>
                  }
               #end
               #local j = j + 1;
            #end
         #end
         #local i = i - 1;
      #end
         
      // ** Door
      union {
         // Posts
         object { Beam(dh) scale 1 + .75*z translate <dw/2, 0 , 0> }
         object { Beam(dh) scale 1 + .75*z translate <-dw/2, 0 , 0> }

         // Crosspost
         object {
            Beam(dw/2) scale 1 + .75*z rotate 90*z 
            translate <0, dh , 0>
         }

         // Wing
         box {
            -1, 1 scale <dw/2, dh, .15>
            texture { T_RoughWood }
         }

         #if (ds = 0)
            translate -d/2 * z
         #end
         #if (ds = 1)
            rotate 90*y
            translate w/2 * x
         #end
         #if (ds = 2)
            translate d/2 * z
         #end
         #if (ds = 3)
            rotate 90*y
            translate -w/2 * x
         #end
      }

      // ** Wattle & Daub main body
      difference {
         box {
            -1, 1
            scale <w/2, h, d/2>
         }
         union {
            box {
               -.9, .9
               scale <w/2, h, d/2>
            }
            #local i = ih;
            #while (i > 0)
               #if (wpy != 0 & mod(i, wpy) = 0)
                  // x-sides
                  #local j = -id;
                  #while (j < id)
                     #if (wpz != 0 & mod(j, wpz) = 0)
                        box {
                           -1, 1 scale <2*w, wh/2, ww/2>
                           translate <0, (i - 1/2)*h/ih, (j + 1/2)*d/(2*id)>
                        }
                     #end
                     #local j = j + 1;
                  #end

                  // z-sides
                  #local j = -iw;
                  #while (j < iw)
                     #if (wpx != 0 & mod(j, wpx) = 0)
                        box {
                           -1, 1 scale <ww/2, wh/2, 2*d>
                           translate <(j + 1/2)*w/(2*iw), (i - 1/2)*h/ih, 0>
                        }
                     #end
                     #local j = j + 1;
                  #end
               #end
               #local i = i - 1;
            #end
         }
         texture { T_WattleDaub }
      }
      prism {
         linear_sweep linear_spline
         -1, 1, 4, <-1, 0>, <1, 0>, <0, 1>, <-1, 0>
         rotate -90*x
         scale <w/2, rh, d/2>
         texture { T_WattleDaub }
         translate <0, h, 0>
      }

      // ** Reet roof
      union {
          prism {
            linear_sweep linear_spline
            -1.1, 1.1, 5,
            <-1.2, .1>, <-1, -.1>, <0, .9>, <0, 1.2>, <-1.2, .1>
            rotate -90*x
            scale <w/2, rh, d/2>
         }
         object {
            O_RoofEnd
            rotate -135*z
            scale <sqrt(2), sqrt(2), 1.1>
            translate -.1*x
            scale <w/2, rh, d/2>
            translate -w/2*x
         }
         texture { T_Reet rotate -atan(2*rh/d) }
         translate <0, h, 0>
      }
      union {
         prism {
            linear_sweep linear_spline
            -1.1, 1.1, 5,
            <1.2, .1>, <1, -.1>, <0, .9>, <0, 1.2>, <1.2, .1>
            rotate -90*x
            scale <w/2, rh, d/2>
         }
         object {
            O_RoofEnd
            rotate -45*z
            scale <sqrt(2), sqrt(2), 1.1>
            translate .1*x
            scale <w/2, rh, d/2>
            translate w/2*x
         }
         texture { T_Reet rotate atan(2*rh/d) }
         translate <0, h, 0>
      }
   }
#end
#end

#declare O_Field = union {
   #local X = 50;
   #local Z = 50;
   #local i = -X;
   #while (i < X)
      #local i = i + 1;
      #local j = -Z;
      #while (j < Z)
         #local j = j + 1;
         cylinder { 0, y, .1
            translate <2*i-1, 0, 2*j-1>
            scale <1/X, .9 + .2*rand(Seed), 1/Z>
            matrix <1, 0, 0, .3*rand(Seed), 1, .3*rand(Seed), 0, 0, 1, 0, 0, 0>
         }
      #end
   #end
   texture { T_Grass }
}

// w = width/2, d = depth/2, h = heigt
#macro Field(w, d, h)
   union {
      #local n = -w/2;
      #while (n < w/2)
         #local n = n + 1;
         #local m = -d/2;
         #while (m < d/2)
            #local m = m + 1;
            object { O_Field translate <2*n, 0, 2*m> }
         #end
      #end
      scale <1, 1, h>
   }
#end

#declare Garden =
#ifndef (DELUX)
blob {
   cylinder { 0, y, 1, 1 translate .55*(x+z) }
   cylinder { 0, y, 1, 1 translate .55*(x-z) }
   cylinder { 0, y, 1, 1 translate .55*(z-x) }
   cylinder { 0, y, 1, 1 translate .55*(-x-z) }
   cylinder { 0, y, 1, 1 } scale 1.3
   pigment { rgb <0, 143/255, 0> }
}
#else
union {
   #local n = -4;
   #while (n < 5)
      union {
         sphere { <.2*rand(Seed)-1.5, 1, 3*n>, 1.5 }
         sphere { <.2*rand(Seed)+1.5, 1, 3*n>, 1.5 }
         pigment { Red } normal { crackle .7 scale .35 }
         translate 6*x
      }
      #local n = n + 1;
   #end
   #local n = -2;
   #while (n < 3)
      sphere {
         <.2*rand(Seed), 2, 5*n>, 2.5
         pigment { Green }
         normal {
            average
            normal_map {
               [ 1 crackle 2 scale 2 ]
               [ 2 bumps .75 ]
            }
         }
      }
      #local n = n + 1;
   #end
   superellipsoid { <.35, .3>
      scale <2, 1.5, 13>
      translate y
      pigment {
         bozo
         color_map {
            [ 0 Green ]
            [ .1 Green ]
            [ .5 rgb <.2, .5, .2> ]
            [ .9 Green ]
            [ 1 Green ]
         }
         scale <1, 1, 5>
      }
      normal {
         average
         normal_map {
            [ 2 crackle 2 scale 2 ]
            [ 1 bumps .75 ]
         }
      }
      translate 11.5*x
   }
   superellipsoid { <.35, .3>
      scale <4, .75, 13>
      translate y
      pigment {
         bozo
         color_map {
            [ 0 Green ]
            [ .1 Green ]
            [ .5 DarkGreen ]
            [ .9 Green ]
            [ 1 Green ]
         }
         scale .35
      }
      normal { crackle scale 4 }
      translate -6*x
   }
   #local n = -5;
   #while (n < 6)
      union {
         cylinder {
            0, 5*y, .2
            pigment { Brown }
         }
         #local m = 0;
         #while (m < 10)
            sphere {
               0, .35
               pigment { DarkOliveGreen }
               translate <.7*sin(m), m*.5, .7*cos(m)>
            }
            #local m = m + 1;
         #end
         translate <-12 - rand(Seed), 0, 2.5*n>
         #local n = n + 1;
      }
   #end
   scale 1/14
   scale .2
}
#end

#macro Tree(lvl, leaf, bark)
   #if (lvl > 0)
      union {
         cone { 0, .15, y, .12 }
         sphere { y, .12 }
         #local i = 1 + 3*rand(Seed);
         #while (i > 0)
            object {
               Tree(lvl - 1, leaf, bark)
               scale .8
               rotate x*(20 + rand(Seed)*20)
               rotate y*rand(Seed)*360
               translate y
            }
            #local i = i - 1;
         #end
         texture { bark }
      }
   #else
      sphere { .5*y, .35 texture { leaf scale 2 } }
   #end
#end

#ifdef (DELUX)
#declare O_Tree = object { Tree(8, T_Grass, T_Wood2) scale 2 }
#else
#declare O_Tree = union {
   sphere { 0, .7 pigment { rgb <0, .7, 0> } }
   cylinder { 0, -(N/HGT-1.5)*y, .15 pigment { rgb <.65, .25, 0> } }
   scale 5
}
#end

#macro RoadPiece(p1, p2)
   union {
      sphere { 0, .25 }
      sphere { VDist(p1,p2)*z, .25 }
      cylinder { 0, z, .25 scale <1, 1, VDist(p1,p2)> }
      scale 1 - .9*y
      Reorient_Trans(z,<(p2-p1).x, 0, (p2-p1).z>)
      Reorient_Trans(<(p2-p1).x, 0, (p2-p1).z>, p2 - p1)
      translate p1
   }
#end

camera {
   location (10*N/HGT)*y - (N/8)*(x+z) look_at (N/2)*(x+z)
}

plane {
   y, -TIDE/HGT
#ifdef (DELUX)
   texture { T_Water }
#else
   pigment { rgb <135/255, 207/255, 1> }
#end
   clipped_by { box { -y, y + N*(x + z) } }
}
