#include <math.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#ifdef __unix__
#include <sys/types.h>
#include <sys/wait.h>
#include <unistd.h>
#endif

#define min(a,b) ((a) < (b) ? (a) : (b))
#define max(a,b) ((a) > (b) ? (a) : (b))
#define crop(a) min(N,max(0,a))
#define dist(a,b) ((a[0]-b[0])*(a[0]-b[0]) + (a[1]-b[1])*(a[1]-b[1]))
#define disti(a,b) dist(c[(a)],c[(b)])

int DEBUG = 1;

/* Some of these are actually map properties and can be changed. Some */
/* can't be modified with repercussions easily. Don't change PI. */

/* Format of fig2dev output */
char *FORMAT = "png";

/* fig2dev magnification */
int MAGNIFY = 4;

/* Random */
int RAND = 1;

/* Good ole Pi */
float PI;

/* Detail to a power of two */
int LVL = 7;
int N = 1;

/* Water, percentage of whole map */
float WW = .1;

/* Tide elevation, +/- gives slack, drawn as sand */
int TIDE = 5;

/* River depth, elevation */
int RVD = 15;

/* EL lines */
int HGT = 40;

/* Initial corner vertex elevation */
float **p;

/* Ravine width, used for height adjustments for rivers and roads */
int RVN = 20;

/* River end points, only one given. */
int ***r;

/* House sizes. (Similar, but not identical to houeshold size). The number of
   houses is $#NH. Note that some houses may be placed under water - they are
   lost:-) */
int NH[] = { 2,3,4,5,6,2,3,4,5,6,
             2,3,4,5,6,2,3,4,5,6,
             2,3,4,5,6,2,3,4,5,6,
             2,3,4,5,6,2,3,4,5,6 };
int NNH = 40;

/* Number of road ends */
float BLK = 8;

/* housing ground, the larger this gets, the more isolation between houses. */
int HG = 4;

/* Forest, percentage of whole map */
float WD = .45;

/* Assart around man-made items, cleared of wood */
int WAS = 16;

/* Road tendency to flatten surrounding */
float REL = .25;

/* Initial vertex woods */
float **w;

/* Wood index */
int wi;

/* Garden size */
int GS = 8;

/* Total field size */
int TFS = 500;

/* Field length, width */
int FL = 15;
int FW = 3;

/* Field rounding, 0..FW */
int FR = 0;

/* Fieldsides: N,E,S,W */
int FS[] = {1,1,1,1};

/* Road exits */
int **rde;

/* House list */
int **hh = 0;

/* Prohibited areas */
float **hhh;

/* Roads */
int **rd;

/* Gardens */
int ***f;
int **rr;

/* Elevation line heights */
int *EL;
int _EL = 7;

/* Elevation lines */
int ***l;
int *_l;
int **lc;
char ***cp;

/* Fields */
int **af;

/* Wood lines */
int **wl;
int *wlc;
char **wc;

/* --- Prototypes --- */

void generate_elevation(void);
void generate_rivers(void);
void generate_houses(void);
void generate_roads(void);
void generate_elevationlines(void);
void generate_gardens(void);
void generate_fields(void);
void generate_woods(void);
void output(void);

void height(int *hp, int x1, int y1, int x2,int y2, int d);
void river(int **r, int i0, int i2);
void houses(int *nhh, int x1,int y1,int x2,int y2,int nh,int d);
void wood(int *wp, int x1,int y1,int x2,int y2,int d);
void road(int **r, int i0, int i2);
int el(int h, int x, int y);
int wd(int h, int x, int y);

void nextdir(int xa, int ya, int xc, int yc, int *xe, int *ye, int d);
void nextptel(int xo, int yo, int x, int y, int p1, int* xr, int* yr);
void nextptwd(int xo, int yo, int x, int y, int p1, int* xr, int* yr);
void randdir(int xa, int ya, int *xe, int *ye, int d);

char* append(char *orig, char *add);
float angle(int **c, int i, int j);
float mod(float x);
float relate();
int grep_array_eq(int* test, int** list);

int dd(int x);
int ii(int x);
int pow2(float x);

int lgth(int *ptr);
int pop(int** ptr);
int shift(int** ptr);
void push(int **ptr, int v);
void pushp(int ***ptr, int* v);
void splice(int **ptr, int from,int len);
int** splicep(int **ptr, int from,int len);
void unshift(int **ptr, int v);

/* --- Main part --- */

int main(int argc, char* argv[]) {
    /* Constants */
    PI = 4*atan2f(1,1);
    int n,m,k;
    sscanf(argv[1],"%d",&LVL);
    for (n = 0; n < LVL; n++) { N *= 2; }

    /* Elevation */
    p = (float**) calloc(N+1, sizeof(float*));
    for (n = 0; n < N+1; n++) {
        p[n] = (float*) calloc(N+1, sizeof(float));
        for (m = 0; m < N+1; m++) {
            p[n][m] = -2*N;
        }
    }

    /* Elevation lines */
    EL = (int*) calloc(_EL, sizeof(int));
    EL[0] = -TIDE;
    EL[1] = TIDE;
    EL[2] = HGT;
    EL[3] = 2*HGT;
    EL[4] = 3*HGT;
    EL[5] = 4*HGT;
    EL[6] = 5*HGT;

    l = (int***) calloc(_EL, sizeof(int**));
    _l = (int*) calloc(_EL, sizeof(int));
    for (n = 0; n < _EL; n++) {
        _l[n] = 0;
    }
    lc = (int**) calloc(_EL, sizeof(int*));
    for (n = 0; n < _EL; n++) {
        lc[n] = (int*) calloc(1, sizeof(int));
        lc[n][0] = -2*N;
    }
    cp = (char***) calloc(_EL, sizeof(char**));
    for (n = 0; n < _EL; n++) {
        cp[n] = (char**) calloc(1, sizeof(char*));
    }

    /* River */
    r = (int***) calloc(N+1, sizeof(int**));
    for (n = 0; n < N+1; n++) {
        r[n] = (int**) calloc(N+1, sizeof(int*));
        for (m = 0; m < N+1; m++) {
            r[n][m] = (int*) calloc(2, sizeof(int));
        }
    }

    for (n = 0; n < N+1; n++) {
        for (m = 0; m < N+1; m++) {
            for (k = 0; k < 2; k++) {
                r[n][m][k] = -1;
            }
        }
    }

    r[0][0][0] = N/2; r[0][0][1] = 0;
    r[0][N][0] = N/2; r[0][N][1] = N;

    /* Woods */
    w = (float**) calloc(N+1, sizeof(float*));
    for (n = 0; n < N+1; n++) {
        w[n] = (float*) calloc(N+1, sizeof(float));
        for (m = 0; m < N+1; m++) {
            w[n][m] = -2.0*N;
        }
    }

    /* Road ends */
    rde = (int**) calloc(3, sizeof(int*));
    for (n = 0; n < 2; n++) {
        rde[n] = (int*) calloc(2, sizeof(int));
    }
    rde[0][0] = 0; rde[0][1] = N/2;
    rde[1][0] = N; rde[1][1] = N/2;

    /* Roads */
    rd = (int**) calloc(1, sizeof(int*));

    /* Houses */
    hh = (int**) calloc(1, sizeof(int*));

    /* Prohibited locations */
    hhh = (float**) calloc(N+1, sizeof(float*));
    for (n = 0; n < N+1; n++) {
        hhh[n] = (float*) calloc(N+1, sizeof(float));
        for (m = 0; m < N+1; m++)
            hhh[n][m] = 0.0;
    }

    /* Fields */
    af = (int**) calloc(1, sizeof(int*));

    /* Wood lines */
    wl = (int**) calloc(1, sizeof(int*));

    wlc = (int*) calloc(1, sizeof(int));
    wlc[0] = -2*N;
    wc = (char**) calloc(1, sizeof(char*));

    sscanf(argv[2],"%d",&DEBUG);
    sscanf(argv[3],"%d",&RAND);
    sscanf(argv[4],"%f",&WW);
    sscanf(argv[5],"%d",&TIDE);
    sscanf(argv[6],"%d",&HGT);
    sscanf(argv[7],"%d",&TFS);
    sscanf(argv[8],"%d",&FL);
    sscanf(argv[9],"%d",&FW);
    sscanf(argv[10],"%d",&FR);
    sscanf(argv[11],"%d",&GS);
    sscanf(argv[12],"%d",&RVD);
    sscanf(argv[13],"%d",&RVN);
    sscanf(argv[14],"%d",&HG); if (HG < 4) HG = 4;
    sscanf(argv[15],"%f",&WD);
    sscanf(argv[16],"%f",&REL);
    sscanf(argv[17],"%f",&BLK);
    sscanf(argv[18],"%d",&WAS);
    sscanf(argv[19],"%d",&NNH); NNH++;
    sscanf(argv[20],"%f",&p[0][0]);
    sscanf(argv[21],"%f",&p[0][N/2]);
    sscanf(argv[22],"%f",&p[0][N]);
    sscanf(argv[23],"%f",&p[N/2][0]);
    sscanf(argv[24],"%f",&p[N/2][N/2]);
    sscanf(argv[25],"%f",&p[N/2][N]);
    sscanf(argv[26],"%f",&p[N][0]);
    sscanf(argv[27],"%f",&p[N][N/2]);
    sscanf(argv[28],"%f",&p[N][N]);
    sscanf(argv[29],"%f",&w[0][0]);
    sscanf(argv[30],"%f",&w[0][N/2]);
    sscanf(argv[31],"%f",&w[0][N]);
    sscanf(argv[32],"%f",&w[N/2][0]);
    sscanf(argv[33],"%f",&w[N/2][N/2]);
    sscanf(argv[34],"%f",&w[N/2][N]);
    sscanf(argv[35],"%f",&w[N][0]);
    sscanf(argv[36],"%f",&w[N][N/2]);
    sscanf(argv[37],"%f",&w[N][N]);
    sscanf(argv[38],"%d",&r[0][0][0]);
    sscanf(argv[39],"%d",&r[0][0][1]);
    sscanf(argv[40],"%d",&r[0][N][0]);
    sscanf(argv[41],"%d",&r[0][N][1]);
    sscanf(argv[42],"%d",&r[1][0][0]);
    sscanf(argv[43],"%d",&r[1][0][1]);
    sscanf(argv[44],"%d",&r[1][N][0]);
    sscanf(argv[45],"%d",&r[1][N][1]);
    sscanf(argv[46],"%d",&rde[0][0]);
    sscanf(argv[47],"%d",&rde[0][1]);
    sscanf(argv[48],"%d",&rde[1][0]);
    sscanf(argv[49],"%d",&rde[1][1]);
    for (n = 0; n < NNH; n++)
        sscanf(argv[50 + n],"%d",&NH[n]);

#ifdef __unix__
    int p1,p2,st;
    if ((p1 = fork()) > 0) {
        if ((p2 = fork()) > 0) {
            int p = wait(&st);
            if (p == p2) {
                fprintf(stderr,"Stuck new makemap, params:");
                for (n = 1; n < argc; n++)
                    fprintf(stderr,"%s ",argv[n]);
                fprintf(stderr,"\n");
            }
            kill(p1,1); kill(p2,1);
        }
        else
            sleep(20);
        return 0;
    }
#endif
    srand(RAND);

    generate_elevation();
    generate_rivers();
    generate_roads();
    generate_houses();
    generate_elevationlines();
    generate_gardens();
    generate_fields();
    generate_woods();
    output();
    return 0;
}

/* --- generator functions --- */

void generate_elevation(void) {
    if (DEBUG) fprintf(stderr,"Generating Elevations...\n");

    if (p[0][0] == -2*N && p[0][N/2] == -2*N && p[0][N] == -2*N &&
        p[N/2][0] == -2*N && p[N/2][N/2] == -2*N && p[N/2][N] == -2*N &&
        p[N][0] == -2*N && p[N][N/2] == -2*N && p[N][N] == -2*N) {
        int i,j;
        for (i = 0; i < N+1; i++)
            for (j = 0; j < N+1; j++)
                p[i][j] = 2*TIDE;
        return;
    }

    // Correct default
    if (p[0][0] == -2*N) p[0][0] = 0;
    if (p[0][N/2] == -2*N) p[0][N/2] = 0;
    if (p[0][N] == -2*N) p[0][N] = 0;
    if (p[N/2][0] == -2*N) p[N/2][0] = 0;
    if (p[N/2][N/2] == -2*N) p[N/2][N/2] = 0;
    if (p[N/2][N] == -2*N) p[N/2][N] = 0;
    if (p[N][0] == -2*N) p[N][0] = 0;
    if (p[N][N/2] == -2*N) p[N][N/2] = 0;
    if (p[N][N] == -2*N) p[N][N] = 0;

    int *hp = (int*) calloc(2*N+1,sizeof(int));

    /* Heights are 0 to N (starting with N/2) */
    height(hp, 0, 0, N, N, N/2);

    int hi = 0;
    float s = 0,S = 0;
    int i,j;
    for (i = 0; i < 2*N+1; i++) { S += hp[i]; }
    for (i = 0; s < S*WW;) {
        s += hp[hi++];
    }
    for (i = 0; i < N+1; i++) {
        for (j = 0; j < N+1; j++) {
            p[i][j] -= (hi - N);
        }
    }
}

void generate_rivers(void) {
    if (DEBUG) fprintf(stderr,"Generating Rivers...\n");
    int i,j,r0,rr;
    for (i = 0; i < N+1; i++) {
        if (r[i][0][0] != -1) river(r[i],0,N);
    }

    if (DEBUG) fprintf(stderr, "Adjusting Elevations for Rivers...\n");
    
    float **dr = (float**) calloc(N+1, sizeof(float*));
    for (i = 0; i < N+1; i++) {
        dr[i] = (float*) calloc(N+1, sizeof(float));
        for (j = 0; j < N+1; j++) {
            dr[i][j] = RVN * RVN;
        }
    }
    
    /* create ravines */
    
    for (rr = 0; rr < N+1; rr++) {
        if (r[rr][0][0] == -1) continue;
        for (r0 = 0; r0 < N+1; r0++) {
            int *t = r[rr][r0];
            int xm = crop(t[0] - RVN); int xM = crop(t[0] + RVN + 1);
            int ym = crop(t[1] - RVN); int yM = crop(t[1] + RVN + 1);
            for (i = xm; i < xM; i++) {
                for (j = ym; j < yM; j++) {
                    /* calculate distance from point of interest. */
                    float dp = (t[0]-i) * (t[0]-i) + (t[1]-j) * (t[1]-j);
                    dr[i][j] = (dp < dr[i][j]) ? dp : dr[i][j];
                }
            }
        }
    }

    /* Apply ravines */
    
    for (i = 0; i < N+1; i++) {
        for (j = 0; j < N+1; j++) {
            float qr = sqrt(dr[i][j])/RVN;
            p[i][j] = p[i][j] * qr - (1 - qr) * RVD;
        }
    }
}

void generate_houses(void) {
    if (DEBUG) fprintf(stderr, "Generating Houses...\n");

    /* Put houses at real end points (not intermediate road end points) and
       prepare overall random indices for the later houses (rdi) */

    int** rdi = 0;
    int i, j, k, l;
    for (i = 0; rd[i] != 0; i++) {
        for (j = 0; rd[i][2*j] != -2*N; j++);
        int lrdi = 2*j;
        for (j = 2; j < lrdi-2; j++) {
            if (j%2 == 0) {
                int* tmp = (int*) calloc(2, sizeof(int));
                tmp[0] = i; tmp[1] = j;
                pushp(&rdi, tmp);
            }
        }
        for (j = 0; j < lrdi; j += lrdi-2) {
            if (rd[i][j] > 0 &&  rd[i][j] < N &&
                rd[i][j+1] > 0 &&  rd[i][j+1] < N) {
                int dbl = 0;
                for (k = 0; dbl == 0 && rd[k] != 0; k++) {
                    if (k != i) {
                        for (l = 0; dbl == 0 && rd[k][2*l] != -2*N; l++) {
                            if (rd[i][j] == rd[k][2*l] &&
                                rd[i][j+1] == rd[k][2*l+1]) {
                                dbl = 1;
                            }
                        }
                    }
                }
                if (dbl == 0) {
                    int* tmp = (int*) calloc(4, sizeof(int));
                    tmp[0] = rd[i][j];
                    tmp[1] = rd[i][j+1];
                    tmp[2] = 1000 * relate();
                    tmp[3] = 9000 * (dd(10)-1) * PI/180;
                    pushp(&hh, tmp);
                    for (k = -HG; k <= HG; k++) {
                        for (l = -HG; l <= HG; l++) {
                            int mk = min(max(0, rd[i][j]+k), N);
                            int ml = min(max(0, rd[i][j+1]+l), N);
                            hhh[mk][ml] = -1;
                        }
                    }
                }
            }
        }
    }

    /* Put remaining houses while there are still possible endpoints.  We have
       to ignore points which already have a house to close, are out of
       bounds. For those legal, we have to find the closest path to existing
       roads (again ignoring endpoints). */


    int _rdi = 0;
    for (_rdi = 0; rdi != 0 && rdi[_rdi] != 0; _rdi++);
    for (k = 0; hh[k] != 0; k++);

    while (k < NNH && _rdi > 0) {
        int rr = dd(_rdi)-1;
        int* r = rdi[rr];

        int* pt1 = (int*)calloc(2, sizeof(int));
        pt1[0] = rd[r[0]][r[1]];
        pt1[1] = rd[r[0]][r[1]+1];

        int dx = rd[r[0]][r[1]-2] - rd[r[0]][r[1]+2];
        int dy = rd[r[0]][r[1]-1] - rd[r[0]][r[1]+3];
        float dxy = sqrt(dx*dx + dy*dy)/sqrt(2);

        int* pt2 = (int*)calloc(2, sizeof(int));
        int d = 2*dd(2)-3;
        pt2[0] = (int)(pt1[0] + (d*HG*dy)/dxy);
        pt2[1] = (int)(pt1[1] - (d*HG*dx)/dxy);

        if (pt2[0] > 0 && pt2[1] > 0 && pt2[0] < N && pt2[1] < N) {
            int allow = (p[pt2[0]][pt2[1]] >= 2*TIDE);
            if (allow) {
                for (i = -HG; allow && (i < HG); i++) {
                    for (j = -HG; allow && (j < 2*HG); j += 2*HG) {
                        int mi = min(max(0, pt2[0]+i), N);
                        int mj = min(max(0, pt2[1]+j), N);
                        allow = (hhh[mi][mj] >= 0);
                    }
                }
            }

            /* find shortest route to roads. If a shortest road leads to an
               end, we drop the whole location or dist = 0. */

            if (allow) {
                int* mpt = pt1;
                int d = dist(pt1,pt2);
                for (i = 0; allow && rd[i] != 0; i++) {
                    for (j = 0; rd[i][2*j] != -2*N; j++);
                    int len = 2*j;
                    for (j = 0; j < len; j++) {
                        if (j%2 == 0) {
                            int* tmp = (int*) calloc(2, sizeof(int));
                            tmp[0] = rd[i][j];
                            tmp[1] = rd[i][j+1];
                            if (d > dist(pt2, tmp)) {
                                mpt = tmp;
                                d = dist(pt2, tmp);
                                allow = (j != 0 && j != len-2 && d != 0);
                            }
                        }
                    }
                }
                if (allow) {
                    int rl = pow2(sqrt((mpt[0]-pt2[0])*(mpt[0]-pt2[0]) +
                                  (mpt[1]-pt2[1])*(mpt[1]-pt2[1])));
                    int** gr = (int**) calloc(rl+1, sizeof(int*));
                    gr[0] = mpt;
                    gr[rl] = pt2;
                    road(gr, 0, rl);
                    int* grt = 0;
                    for (j = 0; j < rl+1; j++) {
                        push(&grt, gr[j][0]);
                        push(&grt, gr[j][1]);
                    }
                    pushp(&rd, grt);

                    int* tmp = (int*)calloc(4, sizeof(int));
                    tmp[0] = pt2[0];
                    tmp[1] = pt2[1];
                    tmp[2] = 1000 * relate();
                    tmp[3] = 9000 * (dd(10)-1) * PI/180;
                    pushp(&hh, tmp);
                    for (i = -HG; i <= HG; i++) {
                        for (j = -HG; j <= HG; j++) {
                            int mi = min(max(0, pt2[0]+i), N);
                            int mj = min(max(0, pt2[1]+j), N);
                            hhh[mi][mj] = -1;
                        }
                    }
                }
            }
        }
        rdi = splicep(rdi, rr, 1);
        for (_rdi = 0; rdi[_rdi] != 0; _rdi++);
        for (k = 0; hh[k] != 0; k++);
    }
}

float relate() {
    return dd(3) == 1 ? .5 : dd(2) == 1 ? 1 : 2;
}

void generate_roads(void) {
    if (DEBUG) fprintf(stderr, "Generating Roads...\n");

    /* Create end points */

    int i,j,k,l,m,n;
    for (i = 0; i < BLK; i++) {
        for (j = 0; j < 3; j++) {
            int px = dd(N);
            int py = dd(N);
            if (p[px][py] > 2*TIDE) {
                int* tmp = (int*) calloc(2, sizeof(int));
                tmp[0] = px; tmp[1] = py;
                pushp(&rde, tmp);
                j = 3; // last;
            }
        }
    }

    /* Add road end points */

    for (i = 0; rde[i] != 0; i++) {
        for (j = 0; hh[j] != 0; j++);
        hh = realloc(hh,(j+2)*sizeof(int*));
        j++;
        while (j > 0) hh[j] = hh[--j];
        hh[0] = rde[i];
    }

    for (j = 0; hh[j] != 0; j++);
    int **c = calloc(j+2,sizeof(int*));
    for (j = 0; hh[j] != 0; j++) {
        int *tmp = calloc(2,sizeof(int));
        tmp[0] = hh[j][0];
        tmp[1] = hh[j][1];
        c[j] = tmp;
    }

    /* The aim is to obtain a minimizing connecting graph. Real life as well
       as this algorithm take some liberty. */
    
    /* The first step will half the number of unconnected subgraphs in each
       iteration finally yielding one big graph. ww is a collection holding
       the individual roads, whereas cr holds the connectivity regions
       only. d0 and d1 hold "closest" indices. */
    
    /* @c contains all house coordinates at this point, and will later hold
       all vertices in the graph. @cr will be the connectivity region for each
       house. */
    
    int **cr = calloc(j,sizeof(int*));
    int *d0 = calloc(j,sizeof(int));
    int **d1 = calloc(j,sizeof(int*));

    for (i = 0; hh[i] != 0; i++) {
        /* Initially each house is connected with itself. */
        cr[i] = calloc(2,sizeof(int));
        cr[i][0] = i;
        cr[i][1] = -2*N;
        d1[i] = calloc(2,sizeof(int));
    }
    
    /* As long as the graph has more than one connectivity region, try and
       merge them. */
    
    int **ww = calloc(i+1,sizeof(int*));
    int _cr = i;
    for (i = 0; i < _cr; i++) {
        ww[i] = calloc(1,sizeof(int));
        ww[i][0] = -2*N;
    }

    while (_cr > 1) {
        /* The first part consists in finding the closest other region for
           each region and the houses which are closest in these subgraphs. */
        for (i = 0; i < _cr; i++) {
            /* d0 contains the index of closest house in the own region */
            d0[i] = cr[i][0];
            
            /* [0] = Index of closest house in, [1] = Index of closest foreign
               subgraph. */
            d1[i][0] = cr[(i+1)%(_cr)][0];
            
            for (j = 0; j < _cr; j++) {
                if (i == j) continue;
                for (l = 0; cr[i][l] != -2*N; l++) {
                    for (k = 0; cr[j][k] != -2*N; k++) {
                        if (disti(cr[j][k],cr[i][l]) > disti(d1[i][0],d0[i]))
                            continue;
                        d0[i] = cr[i][l];
                        d1[i][0] = cr[j][k];
                        d1[i][1] = j;
                    }
                }
            }
        }

        /* In the second part we connect each of these two houses (joining two
           subgraphs) with a road (in symmetric fashion). */
        for (i = 0; i < _cr; i++) {
            for (j = 0; ww[d0[i]][j] != -2*N; j++) {
                if (d1[i][0] == ww[d0[i]][j])
                    break;
            }
            if (ww[d0[i]][j] == -2*N) {
                push(&ww[d0[i]], d1[i][0]);
            }

            for (j = 0; ww[d1[i][0]][j] != -2*N; j++) {
                if (d0[i] == ww[d1[i][0]][j])
                    break;
            }
            if (ww[d1[i][0]][j] == -2*N) {
                push(&ww[d1[i][0]], d0[i]);
            }
        }

        /* The third part merges two subgraphs into one. Since several
           subgraph mergers may take place in this loop, previous merges may
           obsolete upcoming ones. All references to the second subgraphs must
           be moved into a reference in the first. (This must be done for
           subgraph references.) */
        i = 0;
        while (i < _cr) {
            /* Only merge those subgraphs which need it. */
            if (d1[i][1] != i) {
                
                /* Merge the second subgraph to first. */
                for (k = 0; cr[d1[i][1]][k] != -2*N; k++)
                    push(&cr[i], cr[d1[i][1]][k]);
                
                /* Remove second subgraph. */
                _cr--;
                free(cr[d1[i][1]]);
                for (k = d1[i][1]; k < _cr; k++) {
                    cr[k] = cr[k+1];
                }
                
                /* Adapt all subgraph indices. Since subgraphs were merged,
                   indices have shifted, old references might refer to a
                   non-existent or wrong subgraph. */
                
                int oi = d1[i][1];
                int ni = (d1[i][1] > i) ? i : i-1;
                d1[i][1] = d1[oi][1];
                for (j = 0; j <= oi; j++) {
                    if (d1[j][1] == oi) d1[j][1] = ni;
                    if (d1[j][1] > oi) d1[j][1]--;
                }
                for (j = oi+1; j <= _cr; j++) {
                    if (d1[j][1] == oi) d1[j-1][1] = ni;
                    if (d1[j][1] > oi) d1[j-1][1] = d1[j][1]-1;
                    if (d1[j][1] < oi) d1[j-1][1] = d1[j][1];
                }
                
                /* Start all over, since totally new subgraphs may now be
                   present. */
                i = 0;
            }
            else {
                i++;
            }
        }
        /* The above loop will terminate, as there is only a finite set of
           graphs to merge. */
    }

    /* The second step will shorten or replace roads, which don't match a
       criterion in a minimal connecting graph: roads can only connect at
       angles greater than 2*PI/3. At a smaller angle, this is heuristically
       reduced. nh denotes the ultimate house, all other vertices are added
       afterwards and can be moved.  Houses themselves may only be connected
       by a single road, to avoid traffic through houses:-) */

    for (i = 0; hh[i] != 0; i++);
    int nh = i;
    
    /* j runs through all indices and fj denotes the latest fixed index. The
       index is meant with respect to the vertices of the graph. */

    int fj = j = 0;
    int I = 0;
    do {
        for (i = 0; ww[j] != 0 && ww[j][i] != -2*N; i++);
        float *a = calloc(i,sizeof(float));

        for (i = 0; ww[j] != 0 && ww[j][i] != -2*N; i++) {
            
            /* Consider all roads from a house; determine their
               angles. Angles are all with respect to the x-axis. */
            a[i] = angle(c,j,ww[j][i]);
            for (k = 0; k < i; k++) {
                
                /* Calculate the angles between different existing
                   roads. Change something, if the angle gets to small. */
                float da = mod(a[i] - a[k]);
                if (fabsf(da) < 7*PI/12) {
                    int oi = ww[j][i];
                    int ok = ww[j][k];

                    if (j < nh) {
                        /* Add a new point, midway between the three points
                           which form the small angle. */
                        for (l = 0; c[l] != 0; l++);
                        c = realloc(c, (l+2)*sizeof(int*));
                        c[l] = calloc(2,sizeof(int));
                        c[l][0] = (c[j][0]+c[ok][0]+c[oi][0])/3;
                        c[l][1] = (c[j][1]+c[ok][1]+c[oi][1])/3;
                        c[l+1] = 0;
                        /* Add a new road */
                        ww = realloc(ww,(l+2)*sizeof(int*));
                        ww[l] = calloc(4,sizeof(int));
                        ww[l][0] = j;
                        ww[l][1] = oi;
                        ww[l][2] = ok;
                        ww[l][3] = -2*N;
                        ww[l+1] = 0;
                        
                        /* Modify old roads */
                        for (m = 0; ww[oi][m] != -2*N; m++) {
                            if (ww[oi][m] == j) ww[oi][m] = l;
                        }

                        for (m = 0; ww[ok][m] != -2*N; m++)
                            if (ww[ok][m] == j) ww[ok][m] = l;

                        for (m = 0; ww[j][m] != -2*N; m++)
                            if (ww[j][m] == oi) ww[j][m] = l;

                        for (m = 0; ww[j][m] != -2*N; m++)
                            if (ww[j][m] == ok) {
                                for (n = m; ww[j][n] != -2*N; n++)
                                    ww[j][n] = ww[j][n+1];
                                m--;
                            }
                    }
                    else {
                        /* Move the vertex such that the angle matches
                           2*PI/3 more closely */
                        c[j][0] -= (fabsf(da)-2*PI/3)/(2*PI/3)*
                            (c[oi][0]+c[ok][0]-2*c[j][0])/2.0;
                        c[j][1] -= (fabsf(da)-2*PI/3)/(2*PI/3)*
                            (c[oi][1]+c[ok][1]-2*c[j][1])/2.0;
                    }
                    
                    /* Something was fixed, all old angles become invalid,
                       so stop the angle calculation at this point */
                    fj = j;
                    break;
                }
            }
            /* ... stop calculating angles if something was fixed. */
            if (fj == j) break;
        }

        for (n = 0; c[n] != 0; n++);
        if (n > 1) j = (j+1)%(n);
        
        /* Do this as a terminating measure. The heuristic and house
           specials may produce endless calculations. */
        I++;
        
        /* This is done until j finally catches up with fj, i.e. until
           one revolution did not produce any change. */
    } while ((j != fj) && (I < 10000));

    /* Make total road index list. */
    
    int **r1 = calloc(1,sizeof(int*));
    int _r1 = 0;
    i = 0;
    while (c[i] != 0) {
        if (ww[i] != 0 && ww[i][0] != -2*N) {
            int x = shift(&ww[i]);
            if (x > i) {
                r1 = realloc(r1,(_r1+2)*sizeof(int*));
                r1[_r1] = calloc(2,sizeof(int));
                r1[_r1][0] = i;
                r1[_r1][1] = x;
                r1[++_r1] = 0;
            }
        }
        else {
            /* Continue with next household, if there are no more roads
               from here. */
            i++;
        }
    }

    if (DEBUG) fprintf(stderr, "Completing Roads...\n");
    
    for (i = 0; r1[i] != 0; i++) {
        int xs = c[r1[i][0]][0];
        int ys = c[r1[i][0]][1];
        int xe = c[r1[i][1]][0];
        int ye = c[r1[i][1]][1];
        int rl = pow2(sqrt((xs-xe)*(xs-xe) + (ys-ye)*(ys-ye)));
        int **gr = calloc(rl+1,sizeof(int*));
        gr[0] = calloc(2,sizeof(int));
        gr[0][0] = xs; gr[0][1] = ys;
        gr[rl] = calloc(2,sizeof(int));
        gr[rl][0] = xe; gr[rl][1] = ye;
        road(gr,0,rl);
        int *grt = 0;
        int _grt = 0;
        for (j = 0; j < rl+1; j++) {
            grt = realloc(grt, (_grt+3)*sizeof(int));
            grt[_grt] = gr[j][0];
            grt[_grt+1] = gr[j][1];
            grt[_grt+2] = -2*N;
            _grt += 2;
        }
        for (j = 0; rd[j] != 0; j++);
        if (_grt > 1) {
            rd = realloc(rd, (j+2)*sizeof(int*));
            rd[j] = grt;
            rd[++j] = 0;
        }
    }
    
    if (DEBUG) fprintf(stderr, "Collapsing Roads...\n");
    
    for (j = 0; rd[j] != 0; j++) {
        int m,i = 0;
        int **lkj = &rd[j];
        while (1) {
            for (m = 0; (*lkj)[2*m] != -2*N; m++);
            int ln = 2*m;
            if (2*i >= ln) break;
            /* If point x and x+2 are equal, remove x+1 and x+2.  (This is
               cyclic.) After removal, the previous point has to be
               rechecked.  Keep 2 point-lines. */
                
            if (ln > 4 && (*lkj)[2*i] == (*lkj)[(2*i+4)%ln] &&
                (*lkj)[2*i+1] == (*lkj)[(2*i+5)%ln]) {
                splice(lkj,2*i,min(2*i+4,ln) - 2*i);
                splice(lkj,0,max(0,2*i+4-ln));
                i = max(i-2, 0);
                continue;
            }
                
            /* If point x and x+1 are equal, remove x.
               (This is cyclic.) After removal, the previous
               point has to be rechecked. */
            
            if (2*i + 2 < ln &&
                (*lkj)[2*i] == (*lkj)[2*i+2] &&
                (*lkj)[2*i+1] == (*lkj)[2*i+3]) {
                splice(lkj,2*i,2);
                i = max(i-1, 0);
                continue;
            }
            /* If three consecutive points are on the border, remove the
               middle.*/
                
            if (((*lkj)[2*i] == 0 &&
                 (*lkj)[(2*i+2)%ln] == 0 &&
                 (*lkj)[(2*i+4)%ln] == 0) ||
                ((*lkj)[2*i+1] == 0 &&
                 (*lkj)[(2*i+3)%ln] == 0 &&
                 (*lkj)[(2*i+5)%ln] == 0) ||
                ((*lkj)[2*i] == N &&
                 (*lkj)[(2*i+2)%ln] == N &&
                 (*lkj)[(2*i+4)%ln] == N) ||
                ((*lkj)[2*i+1] == N &&
                 (*lkj)[(2*i+3)%ln] == N &&
                 (*lkj)[(2*i+5)%ln] == N)) {
                splice(lkj,2*i+2,min(2*i+4,ln)-2*i-2);
                splice(lkj,0,max(0,2*i+4-ln));
                i = max(i-1, 0);
                continue;
            }
            i++;
        }
    }
    
    /* Remove road endpoints once more */

    for (i = 0; rde[i] != 0; i++) {
        for (j = 0; hh[j] != 0; j++)
            hh[j] = hh[j+1];
        hh = realloc(hh,j*sizeof(int*));
    }
}

void generate_elevationlines(void) {
    if (DEBUG) fprintf(stderr, "Generating Elevation lines...\n");
    
    int i,j,k;
    int** q = (int**) calloc(N+1,sizeof(int*));
    for (i = 0; i < N+1; i++) {
        q[i] = (int*) calloc(N+1,sizeof(int));
    }
    for (k = 0; k < _EL; k++) {
        /* Treat all elevation levels */
        int nl = 0;
        for (i = 0; i < N+1; i++) {
            for (j = 0; j < N+1; j++) {
                if (q[i][j] != k+1) {
                    /* If this point has not been treated at this elevation
                       level, do so. */
                    q[i][j] = k+1;
                    
                    int _lkn = 2;
                    if (!el(EL[k],i,j)) {
                        l[k] = realloc(l[k], (nl+1)*sizeof(int*));
                        int *lkn = l[k][nl];
                        /* First/starting point in curve */
                        lkn = (int*) calloc(_lkn+1, sizeof(int));
                        lkn[0] = i;
                        lkn[1] = j;
                        lkn[2] = -2*N;
                    
                        /* old/previous point in curve */
                        int xo = i;
                        int yo = j;
                    
                        /* present point in curve */
                        int x,y;
                        nextptel(xo,yo,i,j,EL[k],&x,&y);
                        while ((i != x || j != y)) {
                            /* while the present point is a point different
                               from the starting point. */
                        
                            /* Mark point as visited and push on curve */
                            q[x][y] = k+1;
                            _lkn += 2;
                            lkn = realloc(lkn, (_lkn+1)*sizeof(int));
                            lkn[_lkn-2] = x;
                            lkn[_lkn-1] = y;
                            lkn[_lkn] = -2*N;
                        
                            /* Get new point and then shift: new -> present ->
                               old */
                            int xn,yn;
                            nextptel(xo,yo,x,y,EL[k],&xn,&yn);
                            xo = x; yo = y;
                            x = xn; y = yn;
                        }
                        /* Ignore trivial curves */
                        if (_lkn > 7)
                            l[k][nl++] = lkn;
                        else
                            free(lkn);
                    }
                }
            }
        }
        _l[k] = nl;
    }

    if (DEBUG) fprintf(stderr, "Collapsing Elevation Lines...\n");
    
    for (k = 0; k < _EL; k++) {
        for (j = 0; j < _l[k]; j++) {
            int i,m;
            int **lkj = &l[k][j];
            for (i = 0; (*lkj)[2*i] != -2*N;) {
                for (m = 0; (*lkj)[2*m] != -2*N; m++);
                int ln = 2*m;

                /* If point x and x+1 are equal, remove x.  (This is
                   cyclic.) After removal, the previous point has to be
                   rechecked. */
                
                if ((*lkj)[2*i] == (*lkj)[(2*i+2)%ln] &&
                    (*lkj)[2*i+1] == (*lkj)[(2*i+3)%ln]) {
                    splice(lkj,2*i,2);
                    i = max(i-1, 0);
                    continue;
                }

                /* If point x and x+2 are equal, remove x+1 and x+2.  (This is
                   cyclic.) After removal, the previous point has to be
                   rechecked. */
                
                if ((*lkj)[2*i] == (*lkj)[(2*i+4)%ln] &&
                    (*lkj)[2*i+1] == (*lkj)[(2*i+5)%ln]) {
                    splice(lkj,2*i,min(2*i+4,ln) - 2*i);
                    splice(lkj,0,max(0,2*i+4-ln));
                    i = max(i-2, 0);
                    continue;
                }
                
                /* If three consecutive points are on the border, remove the
                   middle.*/
                
                if (((*lkj)[2*i] == 0 &&
                     (*lkj)[(2*i+2)%ln] == 0 &&
                     (*lkj)[(2*i+4)%ln] == 0) ||
                    ((*lkj)[2*i+1] == 0 &&
                     (*lkj)[(2*i+3)%ln] == 0 &&
                     (*lkj)[(2*i+5)%ln] == 0) ||
                    ((*lkj)[2*i] == N &&
                     (*lkj)[(2*i+2)%ln] == N &&
                     (*lkj)[(2*i+4)%ln] == N) ||
                    ((*lkj)[2*i+1] == N &&
                     (*lkj)[(2*i+3)%ln] == N &&
                     (*lkj)[(2*i+5)%ln] == N)) {
                    splice(lkj,2*i+2,min(2*i+4,ln)-2*i-2);
                    splice(lkj,0,max(0,2*i+4-ln));
                    i = max(i-1, 0);
                    continue;
                }
                i++;
            }
        }
    }
    
    /* Orient curves */
    
    for (k = 0; k < _EL; k++) {
        free(lc[k]);
        lc[k] = (int*) calloc(_l[k],sizeof(int));
        for (j = 0; j < _l[k]; j++) {
            /* Index to determine orientation of curve */
            int *lkj = l[k][j];
            lc[k][j] = 0;
            
            for (i = 0; lkj[2*i] != -2*N; i++);
            int ln = 2*i;
            
            if (ln > 5) {
                int id = 0;
                for (i = 1; i < ln/2; i++) {
                    if (lkj[(2*i+2)%ln] < lkj[(2*id+2)%ln] ||
                        (lkj[(2*i+2)%ln] == lkj[(2*id+2)%ln] &&
                         lkj[(2*i+3)%ln] < lkj[(2*id+3)%ln])) {
                        id = i;
                    }
                }
                int x1 = lkj[(2*id+0)%ln];
                int y1 = lkj[(2*id+1)%ln];
                int x2 = lkj[(2*id+2)%ln];
                int y2 = lkj[(2*id+3)%ln];
                int x3 = lkj[(2*id+4)%ln];
                int y3 = lkj[(2*id+5)%ln];
                if ((x1-x2)*(y2-y3) > (x2-x3)*(y1-y2)) {
                    lc[k][j] = 1;
                }
            }
        }
    }

    /* Control points */
    
    for (k = 0; k < _EL; k++) {
        free(cp[k]);
        cp[k] = (char**) calloc(_l[k],sizeof(char*));
        for (j = 0; j < _l[k]; j++) {
            if (l[k][j][0] != -2*N) {
                int i;
                /* Close curve */
                for (i = 0; l[k][j][i] != -2*N; i++);
                int ln = i+2;
                l[k][j] = realloc(l[k][j],(ln+1)*sizeof(int));
                l[k][j][ln-2] = l[k][j][0];
                l[k][j][ln-1] = l[k][j][1];
                l[k][j][ln] = -2*N;
                
                /* Control points */
                cp[k][j] = (char*)calloc(8,sizeof(char));
                memcpy(cp[k][j], " 0.000", 7);
                for (i = 1; i < (ln - 2)/2; i++) {
                    int x = l[k][j][2*i];
                    int y = l[k][j][2*i+1];
                    if (x == 0 || y == 0 || x == N || y == N) {
                        cp[k][j] = append(cp[k][j]," 0.000");
                    }
                    else {
                        cp[k][j] = append(cp[k][j]," 1.000");
                    }
                }
                cp[k][j] = append(cp[k][j]," 0.000");
            }
        }
    }
}

void generate_gardens(void) {
    if (DEBUG) fprintf(stderr, "Generating Gardens...\n");

    int i,j;

    rr = (int**) calloc(N+1, sizeof(int*));
    for (i = 0; i < N+1; i++) {
        rr[i] = (int*) calloc(N+1, sizeof(int));
    }
    
    for (j = 0; rd[j] != 0; j++) {
        for (i = 0; rd[j][2*i] != -2*N; i++) {
            int xm = crop(rd[j][2*i]-1); int xM = crop(rd[j][2*i]+1);
            int ym = crop(rd[j][2*i+1]-1); int yM = crop(rd[j][2*i+1]+1);
            int k,l;
            for (k = xm; k <= xM; k++) {
                for (l = ym; l <= yM; l++) {
                    rr[k][l] = 1;
                }
            }
        }
    }

    for (i = 0; hh[i] != 0; i++);
    f = realloc(f, (i+1)*sizeof(int**));
    f[i] = 0;

    for (i = 0; hh[i] != 0; i++) {
        int fs = 1;
        int _f = 2;
        f[i] = (int**) calloc(_f, sizeof(int*));
        f[i][0] = (int*) calloc(2, sizeof(int));
        f[i][0][0] = hh[i][0];
        f[i][0][1] = hh[i][1];
        f[i][1] = 0;
        for (fs = 0; fs < GS; fs++) {
            int x = f[i][0][0]; int y = f[i][0][1];
            int *t = calloc(2,sizeof(int));
            t[0] = x; t[1] = y;
            while (grep_array_eq(t,f[i]) || hhh[t[0]][t[1]] <= -HG/4 ||
                   rr[t[0]][t[1]] == 1 || p[t[0]][t[1]] < TIDE) {
                randdir(t[0],t[1],&(t[0]),&(t[1]),1);
            }
            if (hhh[t[0]][t[1]] > -HG/4 && rr[t[0]][t[1]] != 1 &&
                p[t[0]][t[1]] >= TIDE) {
                f[i] = realloc(f[i], (++_f)*sizeof(int*));
                int *tmp = (int*) calloc(2,sizeof(int));
                tmp[0] = t[0]; tmp[1] = t[1];
                f[i][_f-2] = tmp;
                f[i][_f-1] = 0;
                int k1, k2;
                for (k1 = t[0]-1; k1 <= t[0]+1; k1++) {
                    for (k2 = t[1]-1; k2 <= t[1]+1; k2++) {
                        if (k1 >= 0 && k1 <= N && k2 >= 0 && k2 <= N)
                            rr[k1][k2] = 1;
                    }
                }
            }
        }
        for (j = 0; j < _f-1; j++)
            f[i][j] = f[i][j+1];
        f[i] = realloc(f[i], (--_f)*sizeof(int*));
    }
}

void generate_fields(void) {
    if (DEBUG) fprintf(stderr, "Generating Fields...\n");

    int i,j,k,l;
    for (j = 0; rd[j] != 0; j++) {
        for (i = 0; rd[j][2*i] != -2*N; i++) {
            int x = rd[j][2*i];
            int y = rd[j][2*i+1];
            for (k = -1; k <= 1; k++) {
                for (l = -1; l <= 1; l++) {
                    rr[min(N,max(0,x+k))][min(N,max(0,y+l))] = 1;
                }
            }
        }
    }

    /* ESE, SE, SSE, ..., ENE, E */
    int d[] = { 2,1,     2,2, 1,2,   0,2,  -1,2, -2,2, -2,1, -2,0,
                -2,-1, -2,-2, -1,-2, 0,-2, 1,-2, 2,-2, 2,-1,  2,0 };
    
    /* N, E, S, W */
    int **b = calloc(4,sizeof(int*));

    int b0[] = {1,0,0,0}, b1[] = {0,1,1,0}, b2[] = {1,0,0,1}, b3[] = {0,0,1,0};
    
    b[0] = b0; b[1] = b1; b[2] = b2; b[3] = b3;

    int f1 = 0;
    int fs = 0;

    while (f1 < 1000 && fs < TFS) {
        f1++;
        int k = 0,n,j,k1,k2;
        int k0 = dd(12);

        while (k0 > 0) {
            k0--;
            k++;
            while (FS[k%4] != 1) {
                k++;
            }
        }

        int i = 2*dd(N/4);
        int d1 = (2*(dd(5) + 4*k)) % 32;
        
        /* Find all starting points for a field */
        int *x = calloc(2*FW, sizeof(int));
        int *y = calloc(2*FW, sizeof(int));
        int pd = (d1 + 8) % 32;
        int ii;
        for (ii = 0; ii < 2*FW; ii++) {
            x[ii] = 2*i*b[k%4][0] + N*b[k%4][1]
                - FW*d[d1]
                + d[pd]*ii;
            y[ii] = 2*i*b[k%4][2] + N*b[k%4][3] - FW*d[d1+1] + d[pd+1]*ii;
        }
        
        /* Move on */
        int **ff = calloc(2*FW+1,sizeof(int*));
        int e = 0;
        while (e != 1) {
            for (ii = 0; ii < 2*FW; ii++) {
                if (((d[d1] < 0 && x[ii] >= 0) ||
                     (d[d1] > 0 && x[ii] <= N)) &&
                    ((d[d1+1] < 0 && y[ii] >= 0) ||
                     (d[d1+1] > 0 && y[ii] <= N)) &&
                    (x[ii] > N || x[ii] < 0 ||
                     y[ii] > N || y[ii] < 0 ||
                     rr[x[ii]][y[ii]] != 1 && p[x[ii]][y[ii]] >= 2*TIDE)) {
                    push(&ff[ii],x[ii]);
                    push(&ff[ii],y[ii]);
                    x[ii] += d[d1];
                    y[ii] += d[d1+1];
                }
                else {
                    e = 1;
                    int l0;
                    for (l0 = 0; l0 < ii; l0++) {
                        pop(&ff[l0]); pop(&ff[l0]);
                    }
                    break;
                }
            }
        }
 
        /* Cut to size */
        for (i = 0; i < 2*FW; i++) {
            if (ff[i] == 0) {
                ff[i] = calloc(2,sizeof(int));
                ff[i][0] = ff[i][1] = -2*N;
            }
            while (lgth(ff[i]) >= 2*FL) {
                shift(&ff[i]); shift(&ff[i]);
            }
        }

        /* Round corners */
        for (k = 0; k < FR; k++) {
            for (ii = 0; ii < FR-k; ii++) {
                for (i = 0; ff[i] != 0; i++);
                pop(&ff[k]); pop(&ff[k]);
                shift(&ff[k]); shift(&ff[k]);
                pop(&ff[i-1-k]); pop(&ff[i-1-k]);
                shift(&ff[i-1-k]); shift(&ff[i-1-k]);
            }
        }

        /* Remove outside */
        for (i = 0; i < 2*FW; i++) {
            while (ff[i][0] != -2*N &&
                   (ff[i][lgth(ff[i])-2] < 0 || ff[i][lgth(ff[i])-1] < 0)) {
                int y1 = pop(&ff[i]);
                int x1 = pop(&ff[i]);
                if (ff[i][0] != -2*N &&
                    ff[i][lgth(ff[i])-2] > 0 && ff[i][lgth(ff[i])-1] > 0) {
                    push(&ff[i], (ff[i][lgth(ff[i])-2] + x1)/2);
                    push(&ff[i], (ff[i][lgth(ff[i])-2] + y1)/2);
                }
            }
            while (ff[i][0] != -2*N &&
                   (ff[i][lgth(ff[i])-2] > N || ff[i][lgth(ff[i])-1] > N)) {
                int y1 = pop(&ff[i]);
                int x1 = pop(&ff[i]);
                if (ff[i][0] != -2*N &&
                    ff[i][lgth(ff[i])-2] < N && ff[i][lgth(ff[i])-1] < N) {
                    push(&ff[i], (ff[i][lgth(ff[i])-2] + x1)/2);
                    push(&ff[i], (ff[i][lgth(ff[i])-2] + y1)/2);
                }
            }
            while (ff[i][0] != -2*N &&
                   (ff[i][0] < 0 || ff[i][1] < 0)) {
                int x1 = shift(&ff[i]);
                int y1 = shift(&ff[i]);
                if (ff[i][0] != -2*N &&
                    ff[i][0] > 0 && ff[i][1] > 0) {
                    unshift(&ff[i], (ff[i][1] + y1)/2);
                    unshift(&ff[i], (ff[i][1] + x1)/2);
                }
            }
            while (ff[i][0] != -2*N &&
                   (ff[i][0] > N || ff[i][1] > N)) {
                int x1 = shift(&ff[i]);
                int y1 = shift(&ff[i]);
                if (ff[i][0] != -2*N &&
                    ff[i][0] < N && ff[i][1] < N) {
                    unshift(&ff[i], (ff[i][1] + y1)/2);
                    unshift(&ff[i], (ff[i][1] + x1)/2);
                }
            }
        }

        i = 0;
        while (ff[0] != 0 && ff[0][0] == -2*N && i < 2*FW) {
            free(ff[0]);
            for (k = 0; k < 2*FW-i; k++) {
                ff[k] = ff[k+1];
            }
            i++;
        }

        i = 2*FW;
        while (i > 0 && ff[i] != 0 && ff[i][lgth(ff[i])-1] == -2*N) {
            free(ff[i]);
            ff[i] = 0;
            i--;
        }
        
        for (n = 0; ff[n] != 0; n++);
        if (n-- < FW) continue;
        int fl = max(lgth(ff[0]),lgth(ff[n]));
        
        int **bf = &ff[0];
        int *tf = ff[n];
        
        int l;
        for (l = 1; l < n; l++) {
            if (lgth(ff[l]) > 0) {
                push(bf, ff[l][lgth(ff[l])-2]);
                push(bf, ff[l][lgth(ff[l])-1]);
            }
            fl = max(fl, lgth(ff[l]));
        }

        if (fl < 2*FW) continue;
        for (l = 0; l < lgth(tf)/2; l++) {
            push(bf, tf[lgth(tf)-2*l-2]);
            push(bf, tf[lgth(tf)-2*l-1]);
        }

        for (l = 1; l < n; l++) {
            if (lgth(ff[n - l]) > 0) {
                push(bf, ff[n - l][0]);
                push(bf, ff[n - l][1]);
            }
        }
        
        /* Subtract from todo and subtract field from available space */

        for (i = 0; af[i] != 0; i++);
        af = realloc(af, (i+2)*sizeof(int*));
        af[i] = *bf;
        af[i+1] = 0;

        for (l = 0; ff[l] != 0; l++) {
            for (n = 0; ff[l][n] != -2*N; n += 2) {
                int x1 = ff[l][n]; int y1 = ff[l][n+1];
                int xm = crop(x1 - WAS); int xM = crop(x1 + WAS);
                int ym = crop(y1 - WAS); int yM = crop(y1 + WAS);
                for (i = xm; i <= xM; i++) {
                    for (j = ym; j <= yM; j++) {
                        hhh[i][j] =
                            min(hhh[i][j],(i-x1)*(i-x1) + (j-y1)*(j-y1) - WAS);
                    }
                }
            }
        }

        for (l = 0; ff[l] != 0; l++) {
            for (i = 0; ff[l][2*i] != -2*N; i++);
            fs += i/2;
            for (k = 0; k < 2*i; k += 2) {
                for (k1 = ff[l][k]-1; k1 <= ff[l][k]+1; k1++) {
                    for (k2 = ff[l][k+1]-1; k2 <= ff[l][k+1]+1; k2++) {
                        if (k1 >= 0 && k1 <= N &&
                            k2 >= 0 && k2 <= N)
                            rr[k1][k2] = 1;
                    }
                }
            }
        }
        free(x);
        free(y);
        free(ff);
    }
}

void generate_woods(void) {
    if (DEBUG) fprintf(stderr, "Generating Woods...\n");

    int *wp = (int*) calloc(2*N+1, sizeof(int));
    wood(wp,0,0,N,N,N/2);
    
    if (DEBUG) fprintf(stderr, "Generating Wood Conture...\n");
    
    int h,i,j,k;
    wi = 0;
    int s = 0,S = 0;
    for (i = 0; i < 2*N+1; i++) S += wp[i];
    while (s < S*WD) {
        s += wp[wi++];
    }
    wi -= N/2;

    for (h = 0; rd[h] != 0; h++) {
        for (k = 0; rd[h][2*k] != -2*N; k++) {
            int x = rd[h][2*k]; int y = rd[h][2*k+1];
            int xm = crop(x - WAS); int xM = crop(x + WAS);
            int ym = crop(y - WAS); int yM = crop(y + WAS);
            for (i = xm; i <= xM; i++) {
                for (j = ym; j <= yM; j++) {
                    hhh[i][j] =
                        min(hhh[i][j], (x-i)*(x-i) + (y-j)*(y-j) - WAS);
                }
            }
        }
    }

    for (h = 0; f != 0 && f[h] != 0; h++) {
        for (k = 0; f[h][k] != 0; k++) {
            int x = f[h][k][0]; int y = f[h][k][1];
            int xm = crop(x - WAS); int xM = crop(x + WAS);
            int ym = crop(y - WAS); int yM = crop(y + WAS);
            for (i = xm; i <= xM; i++) {
                for (j = ym; j <= yM; j++) {
                    hhh[i][j] =
                        min(hhh[i][j], (x-i)*(x-i) + (y-j)*(y-j) - WAS);
                }
            }
        }
    }

    for (h = 0; hh[h] != 0; h++) {
        int x = hh[h][0]; int y = hh[h][1];
        int xm = crop(x - WAS); int xM = crop(x + WAS);
        int ym = crop(y - WAS); int yM = crop(y + WAS);
        for (i = xm; i <= xM; i++) {
            for (j = ym; j <= yM; j++) {
                hhh[i][j] =
                    min(hhh[i][j], (x-i)*(x-i) + (y-j)*(y-j) - WAS);
            }
        }
    }

    int **wq = (int**) calloc(N+1, sizeof(int*));
    for (i = 0; i < N+1; i++) {
        wq[i] = (int*) calloc(N+1, sizeof(int));
        for (j = 0; j < N+1; j++)
            wq[i][j] = 0;
    }

    int nl = 0;
    for (i = 0; i < N+1; i++) {
        for (j = 0; j < N+1; j++) {
            if (wq[i][j] != 1) {
                /* If this point has not been treated at this elevation level,
                   do so. */
                wq[i][j] = 1;
                
                int _lkn = 2;
                if (!wd(wi,i,j)) {
                    wl = realloc(wl, (nl+2)*sizeof(int*));
                    wlc = realloc(wlc, (nl+1)*sizeof(int));
                    wl[nl+1] = 0;
                    int* lkn;
                    /* First/starting point in curve */
                    lkn = (int*) calloc(_lkn+1, sizeof(int));
                    lkn[0] = i;
                    lkn[1] = j;
                    lkn[2] = -2*N;
                
                    /* old/previous point in curve */
                    int xo = i;
                    int yo = j;
                
                    /* present point in curve */
                    int x,y;
                    nextptwd(xo,yo,i,j,wi,&x,&y);
                    while ((i != x || j != y)) {
                        /* while the present point is a point different from
                           the starting point */

                        /* Mark point as visited and push on curve */
                        wq[x][y] = 1;
                        _lkn += 2;
                        lkn = realloc(lkn, (_lkn+1)*sizeof(int));
                        lkn[_lkn-2] = x;
                        lkn[_lkn-1] = y;
                        lkn[_lkn] = -2*N;
                    
                        /* Get new point and then shift:
                           new -> present -> old */
                        int xn,yn;
                        nextptwd(xo,yo,x,y,wi,&xn,&yn);
                        xo = x; yo = y;
                        x = xn; y = yn;
                    }
                    /* Ignore trivial curves */
                    if (_lkn > 7) {
                        wl[nl] = lkn;
                        nl++;
                    }
                    else
                        free(lkn);
                }
            }
        }
    }
    free(wq);

    if (DEBUG) fprintf(stderr, "Collapsing Wood Conture...\n");
    
    for (j = 0; wl[j] != 0; j++) {
        int i = 0;
        int **lkj = &wl[j];
        for (i = 0; (*lkj)[2*i] != -2*N;) {
            for (k = 0; (*lkj)[k] != -2*N; k++);
            int ln = k;

            /* If point x and x+1 are equal, remove x.  (This is
               cyclic.) After removal, the previous point has to be
               rechecked. */
	    
            if ((*lkj)[2*i] == (*lkj)[(2*i+2)%ln] &&
                (*lkj)[2*i+1] == (*lkj)[(2*i+3)%ln]) {
                splice(lkj, 2*i, 2);
                i = max(i-1, 0);
                continue;
            }
            
            /* If point x and x+2 are equal, remove x+1 and x+2.  (This is
               cyclic.) After removal, the previous point has to be
               rechecked. */
	    
            if ((*lkj)[2*i] == (*lkj)[(2*i+4)%ln] &&
                (*lkj)[2*i+1] == (*lkj)[(2*i+5)%ln]) {
                splice(lkj, 2*i, min(2*i+4,ln) - 2*i);
                splice(lkj, 0, crop(2*i+4-ln));
                i = max(i-2, 0);
                continue;
            }
            
            /* If three consecutive points are on the border, remove
               the middle. */
            
            if (((*lkj)[2*i] == 0 &&
                 (*lkj)[(2*i+2)%ln] == 0 &&
                 (*lkj)[(2*i+4)%ln] == 0) ||
                ((*lkj)[2*i+1] == 0 &&
                 (*lkj)[(2*i+3)%ln] == 0 &&
                 (*lkj)[(2*i+5)%ln] == 0) ||
                ((*lkj)[2*i] == N &&
                 (*lkj)[(2*i+2)%ln] == N &&
                 (*lkj)[(2*i+4)%ln] == N) ||
                ((*lkj)[2*i+1] == N &&
                 (*lkj)[(2*i+3)%ln] == N &&
                 (*lkj)[(2*i+5)%ln] == N)) {
                splice(lkj,2*i+2,min(2*i+4,ln) - 2*i - 2);
                splice(lkj,0,    max(0,2*i+4-ln));
                i = max(i-1, 0);
                continue;
            }
            i++;
        }
    }
    
    /* Orient curves */
    
    for (j = 0; wl[j] != 0; j++) {
        /* Index to determine orientation of curve */
        int *lkj = wl[j];
        wlc[j] = 0;
        for (k = 0; lkj[k] != -2*N; k++);
        int ln = k;
        int id = 0;
        if (ln > 5) {
            for (i = 1; i < ln/2; i++) {
                if (lkj[(2*i+2)%ln] == lkj[(2*id+2)%ln] &&
                    lkj[(2*i+3)%ln] == lkj[(2*id+3)%ln]) {
                }
                if (lkj[(2*i+2)%ln] < lkj[(2*id+2)%ln] ||
                    (lkj[(2*i+2)%ln] == lkj[(2*id+2)%ln] &&
                     lkj[(2*i+3)%ln] < lkj[(2*id+3)%ln])) {
                    id = i;
                }
            }
            int x1 = lkj[(2*id+0)%ln];
            int y1 = lkj[(2*id+1)%ln];
            int x2 = lkj[(2*id+2)%ln];
            int y2 = lkj[(2*id+3)%ln];
            int x3 = lkj[(2*id+4)%ln];
            int y3 = lkj[(2*id+5)%ln];
            if ((x1-x2)*(y2-y3) > (x2-x3)*(y1-y2)) {
                wlc[j] = 1;
            }
        }
    }

    /* Control points */

    free(wc);
    for (j = 0; wl[j] != 0; j++);
    wc = (char**) calloc(j+1,sizeof(char*));
    wc[j] = '\0';
    for (j = 0; wl[j] != 0; j++) {
        if (wl[j][0] != -2*N) {
            int i;
            /* Close curve */
            for (i = 0; wl[j][i] != -2*N; i++);
            int ln = i+2;
            wl[j] = realloc(wl[j],(ln+1)*sizeof(int));
            wl[j][ln-2] = wl[j][0];
            wl[j][ln-1] = wl[j][1];
            wl[j][ln] = -2*N;
                
            /* Control points */
            wc[j] = calloc(8,sizeof(char));
            memcpy(wc[j]," 0.000",7);
            for (i = 1; i < (ln - 2)/2; i++) {
                int x = wl[j][2*i];
                int y = wl[j][2*i+1];
                if (x == 0 || y == 0 || x == N || y == N) {
                    wc[j] = append(wc[j]," 0.000");
                }
                else {
                    wc[j] = append(wc[j]," 1.000");
                }
            }
            wc[j] = append(wc[j]," 0.000");
        }
    }
}

void output(void) {
    int i,j,k;
    /* height */
    for (i = 0; i < N+1; i++) {
        for (j = 0; j < N+1; j++) {
            printf("%f", p[i][j]);
            if (j != N) printf(",");
        }
        printf("\n");
    }
    /* elevation lines */
    for (i = 0; i < _EL; i++) {
        printf("%d\n", _l[i]-1);
        for (j = 0; j < _l[i]; j++) {
            k = 0;
            while (l[i][j][k] != -2*N) {
                printf("%d", l[i][j][k]);
                if (l[i][j][k+1] != -2*N) printf(",");
                k++;
            }
            printf("\n");
        }
    }
    /* elevation lines orientation */
    for (i = 0; i < _EL; i++) {
        for (j = 0; j < _l[i]; j++) {
            printf("%d", lc[i][j]);
            if (j+1 < _l[i]) printf(",");
        }
        printf("\n");
    }
    /* control points for elevation */
    for (i = 0; i < _EL; i++) {
        for (j = 0; j < _l[i]; j++) {
            printf("%s", cp[i][j] != 0 ? cp[i][j] : "");
            if (j+1 < _l[i]) printf(",");
        }
        printf("\n");
    }
    /* houses */
    for (i = 0; hh[i] != 0; i++);
    printf("%d\n",i-1);
    for (i = 0; hh[i] != 0; i++) {
        printf("%d,%d,%d,%d\n", hh[i][0], hh[i][1], hh[i][2], hh[i][3]);
    }
    /* roads */
    for (i = 0; rd[i] != 0; i++);
    printf("%d\n",i-1);
    for (i = 0; rd[i] != 0; i++) {
        for (j = 0; rd[i][j] != -2*N; j++) {
            printf("%d",rd[i][j]);
            if (rd[i][j+1] != -2*N) printf(",");
        }
        printf("\n");
    }
    /* gardens */
    for (i = 0; f != 0 && f[i] != 0; i++);
    printf("%d\n",i-1);
    for (i = 0; f != 0 && f[i] != 0; i++) {
        for (j = 0; f[i][j] != 0; j++);
        printf("%d\n",j-1);
        for (j = 0; f[i][j] != 0; j++) {
            for (k = 0; k < 2; k++) {
                printf("%d",f[i][j][k]);
                if (k < 1) printf(",");
            }
            printf("\n");
        }
    }
    /* fields */
    for (i = 0; af[i] != 0; i++);
    printf("%d\n",i-1);
    for (i = 0; af[i] != 0; i++) {
        for (j = 0; af[i][j] != -2*N; j++) {
            printf("%d",af[i][j]);
            if (af[i][j+1] != -2*N) printf(",");
        }
        printf("\n");
    }
    /* forbidden fields */
    for (i = 0; i < N+1; i++) {
        for (j = 0; j < N+1; j++) {
            printf("%f", hhh[i][j]);
            if (j != N) printf(",");
        }
        printf("\n");
    }
    /* woods */
    for (i = 0; i < N+1; i++) {
        for (j = 0; j < N+1; j++) {
            printf("%f", w[i][j]);
            if (j != N) printf(",");
        }
        printf("\n");
    }
    /* wood lines */
    for (i = 0; wl != 0 && wl[i] != 0; i++);
    printf("%d\n",i-1);
    for (j = 0; wl[j] != 0; j++) {
        k = 0;
        while (wl[j][k] != -2*N) {
            printf("%d", wl[j][k]);
            if (wl[j][k+1] != -2*N) printf(",");
            k++;
        }
        printf("\n");
    }
    /* wood lines orientation */
    j = 0;
    while (wl[j] != 0) {
        printf("%d", wlc[j]);
        if (wl[j+1] != 0) printf(",");
        j++;
    }
    printf("\n");
    /* wood control points */
    j = 0;
    while (wl[j] != 0) {
        printf("%s", wc[j] != 0 ? wc[j] : "");
        if (wl[j+1] != 0) printf(",");
        j++;
    }
    printf("\n");
    /* wood index */
    printf("%d\n",wi);
}

/* --- helpers --- */

void height(int *hp, int x1, int y1, int x2,int y2, int d) {
    int xt = ii((x1+x2)/2); int yt = y1;
    int xb = ii((x1+x2)/2); int yb = y2;
    int xl = x1; int yl = ii((y1+y2)/2);
    int xr = x2; int yr = ii((y1+y2)/2);
    int xm = ii((x1+x2)/2); int ym = ii((y1+y2)/2);
    
    /* Point t(op) */
    if (p[xt][yt] == -2*N) {
        p[xt][yt] = (p[x1][y1] + p[x2][y1])/2.0 + d*(dd(3)-2);
        hp[(int)p[xt][yt] + N]++;
    }
        
    /* Point b(ottom) */
    if (p[xb][yb] == -2*N) {
        p[xb][yb] = (p[x1][y2] + p[x2][y2])/2.0 + d*(dd(3)-2);
        hp[(int)p[xb][yb] + N]++;
    }
        
    /* Point l(eft) */
    if (p[xl][yl] == -2*N) {
        p[xl][yl] = (p[x1][y1] + p[x1][y2])/2.0 + d*(dd(3)-2);
        hp[(int)p[xl][yl] + N]++;
    }
        
    /* Point l(eft) */
    if (p[xr][yr] == -2*N) {
        p[xr][yr] = (p[x2][y1] + p[x2][y2])/2.0 + d*(dd(3)-2);
        hp[(int)p[xr][yr] + N]++;
    }
        
    /* Point m(iddle) */
    if (p[xm][ym] == -2*N) {
        p[xm][ym] = (p[xt][yt] + p[xb][yb] +
                     p[xl][yl] + p[xr][yr])/4.0 + d*(dd(3)-2);
        hp[(int)p[xm][ym] + N]++;
    }
        
    /* Recursive call if positive difference */
    if (d > 1) {
        height(hp,x1,y1, xm,ym, d/2);
        height(hp,xt,yt, xr,yr, d/2);
        height(hp,xl,yl, xb,yb, d/2);
        height(hp,xm,ym, x2,y2, d/2);
    }
}

void river(int **r, int i0, int i2) {
    int i1 = ii((i0+i2)/2);
    int dx = r[i2][0] - r[i0][0];
    int dy = r[i2][1] - r[i0][1];
    
    float d = (dd(3) - 2)/8.0;
    
    r[i1][0] = ii((r[i0][0] + r[i2][0])/2 - d * dy);
    r[i1][1] = ii((r[i0][1] + r[i2][1])/2 + d * dx);
    
    if (i2 - i1 > 1) {
        river(r, i0, i1);
        river(r, i1, i2);
    }
}

void houses(int *nhh, int x1,int y1,int x2,int y2,int nh,int d) {
    if (nh == 0) return; 
    if (d <= 1) {
        if (nh < 1 || p[x1][y1] < 2*TIDE) return;
        if (x1 < HG || y1 < HG || x1 > N-HG || y1 > N-HG) return;

        hh = realloc(hh, (*nhh + 1)*sizeof(int*));
        hh[*nhh] = (int*) calloc(4,sizeof(int));
        hh[*nhh][0] = x1;
        hh[*nhh][1] = y1;
        hh[*nhh][2] = NH[*nhh]*dd(2);
        /* Orient the house along elevation lines */
        int x,y,xp,yp,xn,yn;
        nextdir(x1,y1,x1,y1,&x,&y,1);
        xp = x; yp = y; xn = yn = -1;

        while (xn != x || yn != y) {
            nextdir(xp,yp,x1,y1,&xn,&yn,1);
            if ((p[xn][yn] - p[x1][y1])*(p[xp][yp] - p[x1][y1]) < 0) {
                break;
            }
            xp = xn; yp = yn;
        }
        hh[*nhh][3] = 1000 *
            (atan2(yn - y1, xn - x1) + atan2(yp - y1, xp - x1))/2;

        (*nhh)++;
        return;
    }
    
    int xt = ii((x1+x2)/2); int yt = y1;
    int xb = ii((x1+x2)/2); int yb = y2;
    int xl = x1; int yl = ii((y1+y2)/2);
    int xr = x2; int yr = ii((y1+y2)/2);
    int xm = ii((x1+x2)/2); int ym = ii((y1+y2)/2);
    
    /* Recursive call if positive difference */
    int h[] = { 0,0,0,0 };
    int i;
    for (i = 0; i < nh;) {
        float r = (1 - BLK) + BLK*nh;
        int j = dd(4) - 1;
        r = min(r,d/4.0 - h[j]);
        r = min(r,nh - i);
        h[j] += (int)(r + .5);
        i += (int)(r + .5);
    }
        
    houses(nhh, x1,y1, xm,ym, h[0], d/4);
    houses(nhh, xt,yt, xr,yr, h[1], d/4);
    houses(nhh, xl,yl, xb,yb, h[2], d/4);
    houses(nhh, xm,ym, x2,y2, h[3], d/4);
}

void wood(int *wp, int x1,int y1,int x2,int y2,int d) {
    int xt = ii((x1+x2)/2); int yt = y1;
    int xb = ii((x1+x2)/2); int yb = y2;
    int xl = x1; int yl = ii((y1+y2)/2);
    int xr = x2; int yr = ii((y1+y2)/2);
    int xm = ii((x1+x2)/2); int ym = ii((y1+y2)/2);
        
    /* Point t(op) */
    if (w[xt][yt] < 1-2.0*N) {
        w[xt][yt] = (w[x1][y1] + w[x2][y1])/2 + d*(dd(2)-1);
        wp[(int)w[xt][yt] + N/2]++;
    }
        
    /* Point b(ottom) */
    if (w[xb][yb] < 1-2.0*N) {
        w[xb][yb] = (w[x1][y2] + w[x2][y2])/2 + d*(dd(2)-1);
        wp[(int)w[xb][yb] + N/2]++;
    }
        
    /* Point l(eft) */
    if (w[xl][yl] < 1-2.0*N) {
        w[xl][yl] = (w[x1][y1] + w[x1][y2])/2 + d*(dd(2)-1);
        wp[(int)w[xl][yl] + N/2]++;
    }
        
    /* Point l(eft) */
    if (w[xr][yr] < 1-2.0*N) {
        w[xr][yr] = (w[x2][y1] + w[x2][y2])/2 + d*(dd(2)-1);
        wp[(int)w[xr][yr] + N/2]++;
    }
        
    /* Point m(iddle) */
    if (w[xm][ym] < 1-2.0*N) {
        w[xm][ym] = (w[xt][yt] + w[xb][yb] +
                     w[xl][yl] + w[xr][yr])/4 + d*(dd(2)-1);
        wp[(int)w[xm][ym] + N/2]++;
    }
        
    /* Recursive call if positive difference */
    if (d > 1) {
        wood(wp, x1,y1, xm,ym, d/2);
        wood(wp, xt,yt, xr,yr, d/2);
        wood(wp, xl,yl, xb,yb, d/2);
        wood(wp, xm,ym, x2,y2, d/2);
    }

}
    
void road(int **r, int i0, int i2) {
    if (i2 - i0 <= 1) return;

    int i1 = ii((i0+i2)/2);
    int dx = r[i2][0] - r[i0][0];
    int dy = r[i2][1] - r[i0][1];
    
    int **rr = (int**)calloc(3,sizeof(int*));
    int i;
    for (i = -1; i <= 1; i++) {
        rr[1+i] = calloc(2,sizeof(int));
        rr[1+i][0] = ii((r[i0][0] + r[i2][0])/2.0 + i/8.0 * dy);
        rr[1+i][1] = ii((r[i0][1] + r[i2][1])/2.0 + i/8.0 * dx);
    }
    
    int d0 = dd(3);
    if (r[i1] == 0)
        r[i1] = calloc(2,sizeof(int));
    r[i1][0] = rr[d0-1][0];
    r[i1][1] = rr[d0-1][1];
    
    if (i1 - i0 > 1) {
        road(r, i0, i1);
    }
    if (i2 - i1 > 1) {
        road(r, i1, i2);
    }
}

int el(int h, int x, int y) { return (p[x][y] >= h); }

int wd(int h, int x, int y) {
    if (w[x][y] >= h || p[x][y] < TIDE) return 1;
    if (hhh[x][y] < 0) return 1;
    return 0;
}

/* --- directions --- */

int dir[] = { +1,0, +1,+1, 0,+1, -1,+1, -1,0, -1,-1, 0,-1, +1,-1 };

void nextdir(int xo, int yo, int x, int y, int *xe, int *ye, int n) {

    short t = 0;

    if (x != xo || y != yo) {
	while ((x+n*dir[t] != xo || y+n*dir[t+1] != yo)) {
	    t = (t+2) & 15;
	}
    }
    while ((x+n*dir[t] == xo && y+n*dir[t+1] == yo) ||
	   (x+n*dir[t] < 0 || y+n*dir[t+1] < 0 ||
	    y+n*dir[t+1] > N || x+n*dir[t] > N)) {
	t = (t+2) & 15;
    }
    *xe = x+n*dir[t];
    *ye = y+n*dir[t+1];
}

void nextptel(int xo, int yo, int x, int y, int p1, int* xr, int* yr) {
    /* This function finds the next legal point, that is, it circles around
       the present in predefined manner. Once it finds a point that is of
       right elevation, this is returned.
    */

    /* Get first new point */
    int xn,yn;
    nextdir(xo,yo,x,y,&xn,&yn,1);

    /* Hold old vector as test vector. This is the new, if the original was
       trivial */
    int xt = xo;
    int yt = yo;
    if (xo == x && yo == y) {
        xt = xn; yt = yn;
    }

    while ( el(p1,xn,yn) && ((xn != xt) || (yn != yt))) {
        /* Loop, as long as elevation does not match and the test vector isn't
           reached, i.e. one revolution is performed. */
	xo = xn; yo = yn;
	nextdir(xo,yo,x,y,&xn,&yn,1);
    }
    
    /* At this point the elevation is matched or a revolution was
       performed. In the latter case return the center, if the revolution
       wasn't successful. */
    if (el(p1,xn,yn)) {
        xn = x; yn = y;
    }
    *xr = xn; *yr = yn;
}

void nextptwd(int xo, int yo, int x, int y, int p1, int* xr, int* yr) {
    /* This function finds the next legal point, that is, it circles around
       the present in predefined manner. Once it finds a point that is of
       right elevation, this is returned.
    */

    /* Get first new point */
    int xn,yn;
    nextdir(xo,yo,x,y,&xn,&yn,1);

    /* Hold old vector as test vector. This is the new, if the original was
       trivial */
    int xt = xo;
    int yt = yo;
    if (xo == x && yo == y) {
        xt = xn; yt = yn;
    }

    while ( wd(p1,xn,yn) && ((xn != xt) || (yn != yt))) {
        /* Loop, as long as elevation does not match and the test vector isn't
           reached, i.e. one revolution is performed. */
	xo = xn; yo = yn;
	nextdir(xo,yo,x,y,&xn,&yn,1);
    }
    
    /* At this point the elevation is matched or a revolution was
       performed. In the latter case return the center, if the revolution
       wasn't successful. */
    if (wd(p1,xn,yn)) {
        xn = x; yn = y;
    }
    *xr = xn; *yr = yn;
}

void randdir(int xa, int ya, int *xe, int *ye, int d) {
    int r = dd(8);
    int x = xa;
    int y = ya;

    while (r--) {
        nextdir(xa,ya,x,y,&xa,&ya,1);
    }
    *xe = xa;
    *ye = ya;
}

/* --- trivia --- */

char* append(char *orig, char *add) {
    int n = 0, m = 0;
    while (orig && orig[++n]);
    while (add && add[++m]);
    orig = realloc(orig, (n+m+1)*sizeof(char));
    strcpy(orig + n,add);
    return orig;
}

float angle (int **c, int i, int j) {
    int x1 = c[i][0];
    int x2 = c[j][0];
    int y1 = c[i][1];
    int y2 = c[j][1];
    return atan2f(x2 - x1, y2 - y1);
}

float mod(float x) {
    while (x > PI) x -= 2*PI;
    while (x > PI) x += 2*PI;
    return x;
}

int grep_array_eq(int* test, int** list) {
    int i;
    for (i = 0; list[i] != 0; i++) {
        if (list[i][0] == test[0] && list[i][1] == test[1]) {
            return 1; }
    }
    return 0;
}

int dd(int x) { return 1+(int) ((float)(x) * rand()/(RAND_MAX+1.0)); }

int ii(int x) { return (int)(x + .5); }

int lgth(int *ptr) {
    int i = 0;
    if (ptr != 0) for (i = 0; ptr[i] != -2*N; i++);
    return i;
}

int pow2(float x) {
    int r;
    for (r = 2; r < x; r *= 2);
    return r/2;
}

int pop(int **ptr) {
    if (*ptr == 0) return -2*N;
    int i;
    for (i = 0; (*ptr)[i] != -2*N; i++);
    if (i > 0) {
        int ret = (*ptr)[i-1];
        (*ptr)[i-1] = -2*N;
        return ret;
    }
    return -2*N;
}

int shift(int **ptr) {
    if (*ptr == 0) return -2*N;
    int j,i;
    for (i = 0; (*ptr)[i] != -2*N; i++);
    int ret = (*ptr)[0];
    for (j = 0; j < i-1; j++) (*ptr)[j] = (*ptr)[j+1];
    if (i > 0) (*ptr)[i-1] = -2*N;
    return ret;
}

void push(int **ptr, int v) {
    int i;
    for (i = 0; *ptr != 0 && (*ptr)[i] != -2*N; i++);
    *ptr = realloc(*ptr, (i+2)*sizeof(int));
    (*ptr)[i] = v;
    (*ptr)[i+1] = -2*N;
}

void pushp(int ***ptr, int* v) {
    int i;
    for (i = 0; *ptr != 0 && (*ptr)[i] != 0; i++);
    *ptr = realloc(*ptr, (i+2)*sizeof(int*));
    (*ptr)[i] = v;
    (*ptr)[i+1] = 0;
}

void splice(int **ptr, int from,int len) {
    int i;
    for (i = 0; (*ptr)[i] != -2*N; i++);
    int ln = i;
    int *nptr = calloc(max(0, ln - len) + 1, sizeof(int));
    for (i = 0; i < from; i++)
        nptr[i] = (*ptr)[i];
    for (i = from + len; i <= ln; i++)
        nptr[i - len] = (*ptr)[i];
    if (ln < len) nptr[0] = -2*N;
    free(*ptr);
    *ptr = nptr;
}

int** splicep(int **ptr, int from,int len) {
    int i;
    for (i = 0; ptr[i] != 0; i++);
    int ln = i;
    int **nptr = (int**)calloc(max(0, ln - len) + 1, sizeof(int*));
    for (i = 0; i < from; i++)
        nptr[i] = ptr[i];
    for (i = from + len; i <= ln; i++)
        nptr[i - len] = ptr[i];
    if (ln < len) nptr[0] = 0;
    free(ptr);
    return nptr;
}

void unshift(int **ptr, int v) {
    int i,j;
    for (i = 0; (*ptr)[i] != -2*N; i++);
    *ptr = realloc(*ptr, (i+2)*sizeof(int));
    for (j = i; j >= 0; j--)
        (*ptr)[j+1] = (*ptr)[j];
    (*ptr)[0] = v;
}

