#!/usr/bin/perl

#    Copyright (C) 2003, 2004 Michael Jung
#    e-mail: miju@phantasia.org
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.

use Fcntl qw(:flock);
use CGI;
use CGI::Carp qw(fatalsToBrowser);
use GD;
GD::Image->trueColor(1);

$| = 1;

$local = 0;

#########
### INPUT
#########

$q = new CGI;

############
## CONSTANTS
############

# This is for debugging checking how much time the whole program takes.

$t0 = time();

# Maximum allowed time in seconds

$TMAX = 30;

#$DEBUG = 0;
$DEBUG = 1;

# Some of these are actually map properties and can be changed. Some
# can't be modified with repercussions easily. Don't change PI.

# Format of fig2dev output

$FORMAT = "png";

# fig2dev magnification

$MAGNIFY = 4;

# Random

$RAND = 1;

# Good ole Pi

$PI = 4*atan2(1,1);

# Size of HGF
$HGF_S = 1631;
$HGF_O = 10;

# Detail to a power of two

$LVL = 7;
$N = 2**$LVL;

# Water (percentage of whole map)

$WW = .1;

# Tide elevation (+/- gives slack, drawn as sand)

$TIDE = 5;

# River depth (elevation)

$RVD = 15;

# EL lines

$HGT = 40;

# Initial corner vertex elevation

$p[0][0] = 0;
$p[0][$N] = 0;
$p[$N][0] = 0;
$p[$N][$N] = 0;

# Ravine width, used for height adjustments for rivers and roads

$RVN = 20;

# River end points, only one given.

$r[0][0] = [$N/2,0];
$r[0][$N] = [$N/2,$N];

# House sizes (Similar, but nor identical to houeshold size). The
# number of houses is $#NH. Note that some houses may be placed under
# water - they are lost:-)

@NH = (2,3,4,5,6,2,3,4,5,6,
       2,3,4,5,6,2,3,4,5,6,
       2,3,4,5,6,2,3,4,5,6,
       2,3,4,5,6,2,3,4,5,6);

# Block tendency, 0 = spread evenly, 1 = cluster as close as possible.

$BLK = 15;

# housing ground, the larger this gets, the more isolation between
# houses.

$HG = 4;

# Forest (percentage of whole map)

$WD = .45;

# Assart around man-made items (cleared of wood)

$WAS = 16;

# Road tendency to flatten surrounding

$REL = .25;

# Initial vertex woods

$w[0][0] = 0;
$w[0][$N] = 0;
$w[$N][0] = 0;
$w[$N][$N] = 0;

# Garden size

$GS = 8;

# Total field size

$TFS = 500;

# Field length, width

$FL = 15;
$FW = 3;

# Field rounding (0..FW)

$FR = 0;

# Fieldsides (N,E,S,W)

@FS = (1,1,1,1);

# Road exits

$rde[0] = [0,$N/2];
$rde[1] = [$N,$N/2];

#############
## ARGS
#############

# We don't use this on the web
if ($local) {
    while ($#ARGV >= 0) {
        my $arg = shift(@ARGV);
        if ($arg eq "NH" || $arg eq "FS") {
            @{"$arg"} = split(/,/,shift @ARGV);
        }
        elsif ($arg eq "p" || $arg eq "w") {
            ${"$arg"}[shift @ARGV][shift @ARGV] = shift @ARGV;
        }
        elsif ($arg eq "rde") {
            ${"$arg"}[shift @ARGV] = [shift @ARGV, shift @ARGV];
        }
        elsif ($arg eq "r") {
            ${"$arg"}[shift @ARGV][shift @ARGV] = [shift @ARGV, shift @ARGV];
        }
        elsif ($arg eq "action") {
            $q->param("action", shift @ARGV);
        }
        else { ${"$arg"} = shift @ARGV; }
    }
}

$ACTION = $q->param("action");
$DEBUG = $1 if ($q->param("DEBUG") =~ /^(0|1)$/);
$RAND = $1 if ($q->param("RAND") =~ /^([0-9]{1,5})$/);
$WW = $1/100 if ($q->param("WW") =~ /^(100|[0-9]{1,2})$/);
$TIDE = $1 if ($q->param("TIDE") =~ /^([0-9]{1,2})$/);
$HGT = $1 if ($q->param("HGT") =~ /^([1-9][0-9]{0,2})$/);
$TFS = $1 if ($q->param("TFS") =~ /^([0-9]{1,3})$/);
$FL = $1/2 if ($q->param("FL") =~ /^([1-9][0-9]{0,2})$/);
$FW = $1/4 if ($q->param("FW") =~ /^([1-9][0-9]{0,2})$/);
$FR = $1 if ($q->param("FR") =~ /^([0123])$/);
$GS = $1 if ($q->param("GS") =~ /^([0-9]{1,2})$/);
$RVD = $1 if ($q->param("RVD") =~ /^([1-9][0-9]?)$/);
$RVN = $1 if ($q->param("RVN") =~ /^([1-9][0-9]?)$/);
$HG = $1 if ($q->param("HG") =~ /^([1-9][0-9]?)$/);
$WD = $1/100 if ($q->param("WD") =~ /^(100|[0-9]{1,2})$/);
$REL = $1/100 if ($q->param("REL") =~ /^(100|[0-9]{1,2})$/);
$BLK = $1/100 if ($q->param("BLK") =~ /^(100|[0-9]{1,2})$/);
$WAS = ($1)**2 if ($q->param("WAS") =~ /^([1-9][0-9]{1,2})$/);
@NH = split(",",$q->param("NH")) if ($q->param("NH") =~ /^(?:([1-9][0-9]?),)*([1-9][0-9]?)$/);
@FS = split(",",$q->param("FS")) if ($q->param("FS") =~ /^(?:(0|1),){0,3}(0|1)$/);

$p[0][0] = int($1/256*$N) if ($q->param("P00") =~ /^(-?[0-9]{1,3})$/);
$p[0][$N/2] = int($1/256*$N) if ($q->param("P10") =~ /^(-?[0-9]{1,3})$/);
$p[0][$N] = int($1/256*$N) if ($q->param("P20") =~ /^(-?[0-9]{1,3})$/);
$p[$N/2][0] = int($1/256*$N) if ($q->param("P01") =~ /^(-?[0-9]{1,3})$/);
$p[$N/2][$N/2] = int($1/256*$N) if ($q->param("P11") =~ /^(-?[0-9]{1,3})$/);
$p[$N/2][$N] = int($1/256*$N) if ($q->param("P21") =~ /^(-?[0-9]{1,3})$/);
$p[$N][0] = int($1/256*$N) if ($q->param("P02") =~ /^(-?[0-9]{1,3})$/);
$p[$N][$N/2] = int($1/256*$N) if ($q->param("P12") =~ /^(-?[0-9]{1,3})$/);
$p[$N][$N] = int($1/256*$N) if ($q->param("P22") =~ /^(-?[0-9]{1,3})$/);

$w[0][0] = int($1/256*$N) if ($q->param("W00") =~ /^(-?[0-9]{1,3})$/);
$w[0][$N/2] = int($1/256*$N) if ($q->param("W10") =~ /^(-?[0-9]{1,3})$/);
$w[0][$N] = int($1/256*$N) if ($q->param("W20") =~ /^(-?[0-9]{1,3})$/);
$w[$N/2][0] = int($1/256*$N) if ($q->param("W01") =~ /^(-?[0-9]{1,3})$/);
$w[$N/2][$N/2] = int($1/256*$N) if ($q->param("W11") =~ /^(-?[0-9]{1,3})$/);
$w[$N/2][$N] = int($1/256*$N) if ($q->param("W21") =~ /^(-?[0-9]{1,3})$/);
$w[$N][0] =int($1/256*$N) if ($q->param("W02") =~ /^(-?[0-9]{1,3})$/);
$w[$N][$N/2] = int($1/256*$N) if ($q->param("W12") =~ /^(-?[0-9]{1,3})$/);
$w[$N][$N] = int($1/256*$N) if ($q->param("W22") =~ /^(-?[0-9]{1,3})$/);

$r[0][0][1] = int($1/128*$N) if ($q->param("R0BX") =~ /^([0-9]{1,3})$/);
$r[0][0][0] = int($1/128*$N) if ($q->param("R0BY") =~ /^([0-9]{1,3})$/);
$r[0][$N][1] = int($1/128*$N) if ($q->param("R0EX") =~ /^([0-9]{1,3})$/);
$r[0][$N][0] = int($1/128*$N) if ($q->param("R0EY") =~ /^([0-9]{1,3})$/);
$r[1][0][1] = int($1/128*$N) if ($q->param("R1BX") =~ /^([0-9]{1,3})$/);
$r[1][0][0] = int($1/128*$N) if ($q->param("R1BY") =~ /^([0-9]{1,3})$/);
$r[1][$N][1] = int($1/128*$N) if ($q->param("R1EX") =~ /^([0-9]{1,3})$/);
$r[1][$N][0] = int($1/128*$N) if ($q->param("R1EY") =~ /^([0-9]{1,3})$/);

$rde[0][1] = int($1/128*$N) if ($q->param("RE0X") =~ /^([0-9]{1,3})$/);
$rde[0][0] = int($1/128*$N) if ($q->param("RE0Y") =~ /^([0-9]{1,3})$/);
$rde[1][1] = int($1/128*$N) if ($q->param("RE1X") =~ /^([0-9]{1,3})$/);
$rde[1][0] = int($1/128*$N) if ($q->param("RE1Y") =~ /^([0-9]{1,3})$/);

srand($RAND);

# Block unwanted stuff

$p[0][0] = $p[0][$N/2] = $p[0][$N] =
    $p[$N/2][0] = $p[$N/2][$N/2] = $p[$N/2][$N] =
    $p[$N][0] = $p[$N][$N/2] = $p[$N][$N] = -2*$N
    if ($q->param("ELEVATION") eq "NO");

$r[0][0][0] = $r[0][0][1] =
    $r[0][$N][0] = $r[0][$N][1] =
    $r[1][0][0] = $r[1][0][1] =
    $r[1][$N][0] = $r[1][$N][1] = -1
    if ($q->param("RIVERS") eq "NO");

@NH = () if ($q->param("HOUSES") eq "NO");

$rde[0][0] = $rde[0][1] = $rde[1][0] = $rde[1][1] = -1
    if ($q->param("ROADS") eq "NO");

$GS = 0 if ($q->param("GARDENS") eq "NO");

$TFS = 0 if ($q->param("FIELDS") eq "NO");

# Elevation lines to be drawn

@EL = (-$TIDE, $TIDE, $HGT, 2*$HGT, 3*$HGT, 4*$HGT, 5*$HGT);

#############
## GENERATION
#############

$args = "$LVL $DEBUG $RAND $WW $TIDE $HGT $TFS $FL $FW $FR $GS ".
    "$RVD $RVN $HG $WD $REL $BLK $WAS $#NH ".
    $p[0][0]." ".
    (defined($p[0][$N/2])?$p[0][$N/2]:-2*$N)." ".
    $p[0][$N]." ".
    (defined($p[$N/2][0])?$p[$N/2][0]:-2*$N)." ".
    (defined($p[$N/2][$N/2])?$p[$N/2][$N/2]:-2*$N)." ".
    (defined($p[$N/2][$N])?$p[$N/2][$N]:-2*$N)." ".
    $p[$N][0]." ".
    (defined($p[$N][$N/2])?$p[$N][$N/2]:-2*$N)." ".
    $p[$N][$N]." ".
    $w[0][0]." ".
    (defined($w[0][$N/2])?$w[0][$N/2]:-2*$N)." ".
    $w[0][$N]." ".
    (defined($w[$N/2][0])?$w[$N/2][0]:-2*$N)." ".
    (defined($w[$N/2][$N/2])?$w[$N/2][$N/2]:-2*$N)." ".
    (defined($w[$N/2][$N])?$w[$N/2][$N]:-2*$N)." ".
    $w[$N][0]." ".
    (defined($w[$N][$N/2])?$w[$N][$N/2]:-2*$N)." ".
    $w[$N][$N]." ".
    $r[0][0][0]." ".
    $r[0][0][1]." ".
    $r[0][$N][0]." ".
    $r[0][$N][1]." ".
    (defined($r[1][0][0])?$r[1][0][0]:-1)." ".
    (defined($r[1][0][1])?$r[1][0][1]:-1)." ".
    (defined($r[1][$N][0])?$r[1][$N][0]:-1)." ".
    (defined($r[1][$N][1])?$r[1][$N][1]:-1)." ".
    $rde[0][0]." ".
    $rde[0][1]." ".
    $rde[1][0]." ".
    $rde[1][1];

$args .= " $_" foreach (@NH);
$args .= " $_" foreach (@FS);

print STDERR "$args\n";
open(IN, "./makemap $args |");

# height
foreach (0..$N) { @{$p[$_]} = split(/,/,<IN>); }

# elevation lines
foreach my $i (0..$#EL) {
    my $ll = <IN>;
    foreach my $j (0..$ll) {
        @{$l[$i][$j]} = split(/,/,<IN>);
    }
}
# elevation lines orientation
foreach my $i (0..$#EL) { @{$lc[$i]} = split(/,/,<IN>); }
# control points for elevation
foreach my $i (0..$#EL) { @{$cp[$i]} = split(/,/,<IN>); }
# houses
my $lll = <IN>;
foreach (0..$lll) { @{$hh[$_]} = split(/,/,<IN>); }
# roads
my $lll = <IN>;
foreach (0..$lll) { @{$rd[$_]} = split(/,/,<IN>); }
# gardens
my $lll = <IN>;
foreach my $i (0..$lll) {
    my $ll = <IN>;
    foreach my $j (0..$ll) {
        @{$f[$i][$j]}  = split(/,/,<IN>);
    }
}
# fields
my $lll = <IN>;
foreach (0..$lll) { @{$af[$_]} = split(/,/,<IN>); }
# forbidden fields
foreach (0..$N) { @{$hhh[$_]}  = split(/,/,<IN>); }
# woods
foreach (0..$N) { @{$w[$_]} = split(/,/,<IN>); }
# wood lines
my $lll = <IN>;
foreach my $i (0..$lll) {
    @{$wl[$i]}  = split(/,/,<IN>);
}
# wood orientation
@wlc  = split(/,/,<IN>);
# wood control points
@wc  = split(/,/,<IN>);
# wood index
$wi = <IN>;

#########
## OUTPUT
#########

%head = ( "FIG" => ["application/fig", "map.fig"],
          "PNG" => ["image/png", "map.png"],
          "POV" => ["application/gzip", "map.pov.gz"],
          "HGF" => ["image/png", "height.png"] );

print $q->header(-type => $head{$ACTION}->[0], -attachment => $head{$ACTION}->[1])
    if (!$local);

&output_header();
&output_elevation() if ($q->param("ELEVATION") ne "NO" ||
                        $q->param("RIVERS") ne "NO");
&output_houses() if ($q->param("HOUSES") ne "NO");
&output_roads() if ($q->param("ROADS") ne "NO");
&output_gardens() if ($q->param("GARDENS") ne "NO");
&output_fields() if ($q->param("FIELDS") ne "NO");
&output_woods() if ($q->param("WOODS") ne "NO");

if ($ACTION eq "FIG") {
    print $result;
}
elsif ($ACTION eq "POV") {
    open(OUT,"| gzip");
    print OUT $result;
    close OUT;
}
elsif ($ACTION eq "HGF") {
    binmode OUT;
    print $result->png;
}
else {
    open(OUT,"| /usr/X11R6/bin/fig2dev -L $FORMAT -m $MAGNIFY");
    print OUT $result;
    close OUT;
}

# End of story:

close STDOUT;
print STDERR "Time used: ", time() - $t0 ,"s\n" if ($DEBUG);

# Crop
sub crop { return $N if ($_[0] > $N); return 0 if ($_[0] < 0); return $_[0]; }

# Rounds
sub i { return int ( $_[0] + .5 ); }

sub d { return int(rand($_[0]) + 1); }

sub min { return @_[0] > @_[1] ? @_[1] : @_[0]; }

sub max { return @_[0] < @_[1] ? @_[1] : @_[0]; }

sub line {
    # This method prints the fig primitive
    my ($w, $pc, $fc, $d, $fp) = @_;
    $result .= "3 4 0 $w $pc $fc $d -1 $fp 0.000 1 0 0 ";
}

sub output_header {
    if ($ACTION eq "POV") {
        $result .= "#declare N = $N;\n";
        $result .= "#declare HGT = $HGT;\n";
        $result .= "#declare TIDE = $TIDE;\n";
        open(HEAD,"<header.pov");
        while (<HEAD>) { $result .= $_; }
    }
    elsif ($ACTION eq "HGF") {
        $result = new GD::Image($HGF_S,$HGF_S);
    }
    else {
        open(HEAD,"<header.fig");
        while (<HEAD>) { $result .= $_; }
        
        $result .= "2 3 0 1 20 14 100 -1 20 0.000 0 0 0 0 0 5\n\t";
        $result .= " ".int(45)." ".int(45);
        $result .= " ".int(45)." ".int(45 + $N*45);
        $result .= " ".int(45 + $N*45)." ".int(45 + $N*45);
        $result .= " ".int(45 + $N*45)." ".int(45);
        $result .= " ".int(45)." ".int(45)."\n";
        
        $result .= "2 3 0 3 -1 -1 0 -1 -1 0.000 0 0 0 0 0 5\n\t";
        $result .= " ".int(45)." ".int(45);
        $result .= " ".int(45)." ".int(45 + $N*45);
        $result .= " ".int(45 + $N*45)." ".int(45 + $N*45);
        $result .= " ".int(45 + $N*45)." ".int(45);
        $result .= " ".int(45)." ".int(45)."\n";
    }
}
    
sub output_elevation {
    my $NN = $N/2;
    my @pp;
    foreach my $i (0..$NN) {
        foreach my $j (0..$NN) {
            $pp[$i][$j] = $p[2*$i][2*$j];
        }
    }
    print STDERR "Printing Elevation...\n" if ($DEBUG);
    
    if ($ACTION eq "POV") {
        $result .= "#declare MyHeight = mesh2 {\n";
        $result .= "   vertex_vectors { ".($NN*$NN*4).",\n";
        foreach my $i (1..$NN) {
            foreach my $j (1..$NN) {
                my $p1 = $pp[$i-1][$j-1]/$HGT;
                my $p2 = $pp[$i-1][$j]/$HGT;
                my $p3 = $pp[$i][$j-1]/$HGT;
                my $p4 = $pp[$i][$j]/$HGT;
                $result .= "      <".($i-1).", ".($p1).", ".($NN-$j+1).">,";
                $result .= " <".($i-1).", ".($p2).", ".($NN-$j).">,";
                $result .= " <".($i).", ".($p4).", ".($NN-$j).">,";
                $result .= " <".($i).", ".($p3).", ".($NN-$j+1).">\n";
            }
        }
        $result .= "   }\n   normal_vectors { ".($NN*$NN*4).",\n";
        foreach my $i (1..$NN) {
            foreach my $j (1..$NN) {
                my $p11 = ($pp[$i][$j-1] - $pp[$i-1][$j-1])/$HGT;
                my $p12 = ($pp[$i-1][$j] - $pp[$i-1][$j-1])/$HGT;
                my $p21 = ($pp[$i][$j] - $pp[$i-1][$j])/$HGT;
                my $p22 = ($pp[$i-1][$j+1] - $pp[$i-1][$j])/$HGT;
                my $p31 = ($pp[$i+1][$j-1] - $pp[$i][$j-1])/$HGT;
                my $p32 = ($pp[$i][$j] - $pp[$i][$j-1])/$HGT;
                my $p41 = ($pp[$i+1][$j] - $pp[$i][$j])/$HGT;
                my $p42 = ($pp[$i][$j+1] - $pp[$i][$j])/$HGT;
                $result .= "      <".(-$p12).", -1, $p11>,";
                $result .= " <".(-$p22).", -1, $p21>,";
                $result .= " <".(-$p42).", -1, $p41>,";
                $result .= " <".(-$p32).", -1, $p31>\n";
            }
        }
        $result .= "   }\n   face_indices { ".($NN*$NN*2).",\n";
        foreach my $i (0..$NN*$NN-1) {
            $result .= "      <".(4*$i).",".(4*$i+1).",".(4*$i+2).">,<".(4*$i).",".(4*$i+3).",".(4*$i+2).">\n";
        }
        $result .= "   }\n   normal_indices { ".($NN*$NN*2).",\n";
        foreach my $i (0..$NN*$NN-1) {
            $result .= "      <".(4*$i).",".(4*$i+1).",".(4*$i+2).">,<".(4*$i).",".(4*$i+3).",".(4*$i+2).">\n";
        }
        $result .= "   }\n   scale <2,1,2>\n}\n";
    }
    elsif ($ACTION eq "HGF") {
        foreach my $i (1..$N) {
            foreach my $j (1..$N) {
                my $p4 = $p[$i][$j]/$N*255;
                my $col = $result->colorAllocate($p4,$p4,$p4);
                $result->filledRectangle
                    (($i - 1)*($HGF_S - 2*$HGF_O)/$N,
                     ($j - 1)*($HGF_S - 2*$HGF_O)/$N,
                     $i*($HGF_S - 2*$HGF_O)/$N,
                     $j*($HGF_S - 2*$HGF_O)/$N, $col);
            }
        }
    }
    else {
        foreach my $k (0..$#EL) {
            # All elevation levels
            foreach my $j (0..$#{$l[$k]}) {
                if ($#{$l[$k][$j]} > 0 ) {
                    my ($w, $pc, $fc, $d, $fp);
                    
                    # Handle all non-trivial elevation curves

                    ($w, $pc, $fc, $d, $fp) = (0, -1, 11, 10, 20)
                        if ($k == 0); # water
                    
                    if ($k == 1) { # tide
                        if ($lc[$k][$j] == 0) {
                            ($w, $pc, $fc, $d, $fp) = (0, -1, 14, 69, 20);
                        }
                        else {
                            ($w, $pc, $fc, $d, $fp) = (0, -1, 6, 70, 20);
                        }
                    }
                    
                    ($w, $pc, $fc, $d, $fp) = (1, 20, 14, 30, -1)
                        if ($k > 1); # ground
                    
                    line($w, $pc, $fc, $d, $fp);
                    
                    my $ln = $#{$l[$k][$j]};
                    
                    # Length of curve
                    $result .= (($ln + 1)/2);
                    $result .= "\n\t";
                    
                    # Print points of curve
                    foreach my $i (0..($ln - 1)/2) {
                        $result .= " ".i(45 + ${$l[$k][$j]}[2*$i] * 45);
                        $result .= " ".i(45 + ${$l[$k][$j]}[2*$i+1] * 45);
                    }
                    $result .= "\n\t";
                    
                    # Control points/Fig format
                    $result .= $cp[$k][$j]."\n";
                }
            }
        }
    }
}

sub output_houses {
    print STDERR "Printing Houses...\n" if ($DEBUG);
    
    foreach my $h (0..$#hh) {
        my $i = $hh[$h][0];
        my $j = $hh[$h][1];
        my $r = $hh[$h][2]/1000; # $s1/$s2
        my $a = $hh[$h][3]/1000;
        next if ($r == 0);

        my $s1 = sqrt($NH[$h]*$r)/2;
        my $s2 = sqrt($NH[$h]/$r)/2;

        if ($ACTION eq "POV") {
            my $p = $p[$i][$j]/$HGT;
            $result .= "object {\n";
            $result .= "   House(".(10*$s1).", 2, ".(10*$s2).", 0, 1.1, 1.9, 1, 1, 2, 1, .5, .5)\n";
            $result .= "   scale .2\n";
            $result .= "   translate $p*y\n";
            $result .= "   rotate ".($a*180/$PI)."*y translate $i*x + ".($N-$j)."*z\n}\n";
        }
        elsif ($ACTION eq "HGF") {
        }
        else {
            $result .= "2 1 0 1 0 7 20 0 20 0.000 0 0 -1 0 0 5\n\t";
            $result .= sprintf "%d %d %d %d %d %d\n\t %d %d %d %d\n",
            i(45 + $i*45 - 45*(cos($a)*$s1 + sin($a)*$s2)),
            i(45 + $j*45 - 45*(sin($a)*$s1 - cos($a)*$s2)),
            i(45 + $i*45 - 45*(cos($a)*$s1 - sin($a)*$s2)),
            i(45 + $j*45 - 45*(sin($a)*$s1 + cos($a)*$s2)),
            i(45 + $i*45 + 45*(cos($a)*$s1 + sin($a)*$s2)),
            i(45 + $j*45 + 45*(sin($a)*$s1 - cos($a)*$s2)),
            i(45 + $i*45 + 45*(cos($a)*$s1 - sin($a)*$s2)),
            i(45 + $j*45 + 45*(sin($a)*$s1 + cos($a)*$s2)),
            i(45 + $i*45 - 45*(cos($a)*$s1 + sin($a)*$s2)),
            i(45 + $j*45 - 45*(sin($a)*$s1 - cos($a)*$s2));
        }
    }
}
    
sub output_roads {
    print STDERR "Printing Roads...\n" if ($DEBUG);
    
    if ($ACTION eq "POV") {
        $result .= "#declare MyRoads = union {\n";
        foreach my $j (0..$#rd) {
            foreach my $i (1..($#{$rd[$j]} - 1)/2) {
                my $x1 = $rd[$j][2*$i-2];
                my $z1 = $rd[$j][2*$i-1];
                my $x2 = $rd[$j][2*$i];
                my $z2 = $rd[$j][2*$i+1];
                my $p1 = $p[$x1][$z1]/$HGT;
                my $p2 = $p[$x2][$z2]/$HGT;
                if ($x1 != $x2 || $z1 != $z2) {
                    $result .= "   cylinder { <$x1, $p1, ".($N-$z1).">, "; 
                    $result .= "<$x2, $p2, ".($N-$z2).">, ";
                    $result .= " .25 }\n";
                }
                $result .= "   sphere { <$x1, $p1, ".($N-$z1).">, .25 }\n"
                    if ($i == ($#{$rd[$j]} - 1)/2);
            }
        }
        $result .= "}\n\n";
        $result .= "#ifdef (DELUX)\n";
        $result .= "object { MyHeight clipped_by { plane { y, ".($TIDE/$HGT)." } } texture { T_Sand } }\n";
        $result .= "object { MyHeight clipped_by { plane { -y, -".($TIDE/$HGT)." } } texture { T_Grass } }\n";
        $result .= "#else\n";
        $result .= "object { MyHeight clipped_by { plane { y, ".($TIDE/$HGT)." } } pigment { rgb <1, 1, 0> } }\n";
        $result .= "object { MyHeight clipped_by { plane { -y, -".($TIDE/$HGT)." } } pigment { rgb <0, .82, 0> } }\n";
        $result .= "#end\n";
        $result .= "#ifdef (DELUX)\n";
        $result .= "object { MyHeight translate .01*y clipped_by { ";
        $result .= "object { MyRoads } } texture { T_Road } }\n";
        $result .= "#else\n";
        $result .= "object { MyHeight translate .01*y clipped_by { ";
        $result .= "object { MyRoads } } pigment { rgb <.63, .25, 0> } }\n";
        $result .= "#end\n";
    }
    elsif ($ACTION eq "HGF") {
    }
    else {
        foreach my $j (0..$#rd) {
            my $ln = $#{$rd[$j]};
                
            if ($ln > 3) {
                my ($w, $pc, $fc, $d, $fp) = (2, 25, -1, 40, -1);
                line($w, $pc, $fc, $d, $fp);
                
                # Length of curve
                $result .= (($ln + 1)/2)."\n\t";
                
                my $c = "0.000 ";
                # Print points of curve
                foreach my $i (0..($ln - 1)/2) {
                    $c .= "1.000 " if ($i > 1);
                    $result .= " ".i(45 + $rd[$j][2*$i] * 45);
                    $result .= " ".i(45 + $rd[$j][2*$i+1] * 45);
                }
                $result .= "\n\t";
                
                # Control points/Fig format
                $result .= "$c 0.000\n";
            }
            elsif ($ln > 1) {
                $result .= "2 1 0 2 25 7 40 0 -1 0.000 1 1 -1 0 0 ";
                
                # Length of curve
                $result .= (($ln + 1)/2)."\n\t";
                
                # Print points of curve
                foreach my $i (0..($ln - 1)/2) {
                    $result .= " ".i(45 + $rd[$j][2*$i]*45);
                    $result .= " ".i(45 + $rd[$j][2*$i+1]*45);
                }
                $result .= "\n";
            }
        }
    }
}

sub output_gardens {
    print STDERR "Printing Gardens...\n" if ($DEBUG);

    if ($ACTION eq "POV") {
        $result .= "intersection {\n   union {\n";
        foreach $i (@f) {
            foreach my $j (0..$#{$i}) {
                my $t = ${$i}[$j];
                $result .= "      object { Garden ";
                $result .= "translate <".$t->[0].", ";
                $result .= ($p[$t->[0]][$t->[1]]/$HGT).", ".($N-$t->[1])."> }\n";
            }
        }
        $result .= "   }\n   object { MyHeight translate .2*y }\n";
        $result .= "#ifndef (DELUX)\n";
        $result .= "   pigment { rgb <0, 143/255, 0> }\n";
        $result .= "#end\n}\n";
    }
    elsif ($ACTION eq "HGF") {
    }
    else {
        foreach $i (@f) {
            foreach my $j (0..$#{$i}) {
                $result .= "2 4 0 1 12 12 50 0 20 0.000 0 0 2 0 0 5\n\t ".
                    (45 + i(45*crop(${$i}[$j][0] - 1)))." ".
                    (45 + i(45*crop(${$i}[$j][1] - 1)))." ".
                    (45 + i(45*crop(${$i}[$j][0] - 1)))." ".
                    (45 + i(45*crop(${$i}[$j][1] + 1)))." ".
                    (45 + i(45*crop(${$i}[$j][0] + 1)))." ".
                    (45 + i(45*crop(${$i}[$j][1] + 1)))." ".
                    (45 + i(45*crop(${$i}[$j][0] + 1)))." ".
                    (45 + i(45*crop(${$i}[$j][1] - 1)))." ".
                    (45 + i(45*crop(${$i}[$j][0] - 1)))." ".
                    (45 + i(45*crop(${$i}[$j][1] - 1)))."\n";
            }
        }
    }
}

sub output_fields {
    print STDERR "Printing Fields...\n" if ($DEBUG);
    
    if ($ACTION eq "POV") {
        $result .= "intersection {\n   union {\n";
        foreach $i (0..$#af) {
            my @a = @{$af[$i]};
            $result .= "      prism { 0, $N, ".(($#a + 3)/2);
            foreach my $i (0..($#a - 1)/2) {
                $result .= ", <".$a[2*$i].", ".($N-$a[2*$i+1]).">";
            }
            $result .= ", <".$a[0].", ".($N-$a[1]).">";
            $result .= "\n      }\n";
        }
        $result .= "   }\n   object { MyHeight translate .3*y }\n";
        $result .= "   pigment { rgb <161/255, 63/255, 0> }\n}\n";
    }
    elsif ($ACTION eq "HGF") {
    }
    else {
        foreach $i (0..$#af) {
            my @a = @{$af[$i]};
            my $ln = $#a;
            if ($ln > 3) {
                $result .= "3 1 0 1 25 25 60 -1 20 0.000 1 0 0 ";
                
                # Length of curve
                $result .= (($ln + 1)/2);
                $result .= "\n\t";
                
                my $c;
                # Print points of curve
                foreach my $i (0..($ln - 1)/2) {
                    my $x = $a[2*$i];
                    my $y = $a[2*$i+1];
                    if ($x != 0 && $x != $N && $y != 0 && $y != $N) {
                        $c .= "1.000 ";
                    }
                    else {
                        $c .= "0.000 ";
                    }
                    $result .= " ".i(45 + $x * 45);
                    $result .= " ".i(45 + $y * 45);
                }
                $result .= "\n\t";

                # Control points/Fig format
                $result .= "$c\n";
            }
        }
    }
}

sub output_woods {
    print STDERR "Printing Woods...\n" if ($DEBUG);

    if ($ACTION eq "POV") {
        foreach my $i (0..$N) {
            foreach my $j (0..$N) {
                if ($p[$i][$j] >= $TIDE && $hhh[$i][$j] >= 0 &&
                    $w[$i][$j] <= $wi) {
                    $result .= "object {\n";
                    $result .= "   O_Tree scale .2 rotate ".(d(360))."*y\n";
                    $result .= "   translate <".($i+(d(3)-2)/3).", ";
                    $result .= ($p[$i][$j]/$HGT + 2 - d(3)/3).", ";
                    $result .= ($N-$j+(d(3)-2)/3).">\n}\n";
                }
            }
        }
    }
    elsif ($ACTION eq "HGF") {
    }
    else {
        foreach my $j (0..$#wl) {
            if ($#{$wl[$j]} > 0 ) {
                my ($w, $pc, $fc, $d, $fp);
                
                # Handle all non-trivial wood curves
                if ($wlc[$j] == 0) {
                    ($w, $pc, $fc, $d, $fp) = (3, 13, 14, 79, 20);
                }
                else {
                    ($w, $pc, $fc, $d, $fp) = (3, 13, 13, 80, 20);
                }
                
                line($w, $pc, $fc, $d, $fp);
                
                # Close curve in such a way that the proper side of it
                # is drawn.
                my $ln = $#{$wl[$j]};
                
                # Length of curve
                $result .= (($ln + 1)/2);
                $result .= "\n\t";
                
                # Print points of curve
                foreach my $i (0..($ln - 1)/2) {
                    my $x = ${$wl[$j]}[2*$i];
                    my $y = ${$wl[$j]}[2*$i+1];
                    $result .= " ".i(45 + $x * 45);
                    $result .= " ".i(45 + $y * 45);
                }
                $result .= "\n\t";
                
                # Control points/Fig format
                $result .= $wc[$j]."\n";
            }
        }
    }
}
